import sys
import argparse
import numpy as np
import verif.input
import netCDF4
import copy
import scipy.signal


def calculate_window(array, threshold, interval, leadtimes):
    """ Computes a weather window """
    O = array.shape[1]
    Inan = np.isnan(array)
    for o in range(0, O):
        q = np.nansum(interval.within(np.cumsum(array[:, o:, :], axis=1)), axis=1)
        I = q+o
        I[I >= O] = O-1
        array[:, o, :] = leadtimes[I] - leadtimes[o]
    array[Inan] = np.nan
    return array


def main():
    parser = argparse.ArgumentParser(prog="window", description="Converts observations and forecasts into weather windows: A window is the leng of time that a parameter is below a certain threshold, for example a dry-spell is the length (along the leadtime axis) that precipitation is 0 mm.")
    parser.add_argument('ifile', help="Verif text or NetCDF file (input)")
    parser.add_argument('ofile', help="Verif NetCDF file (output)")
    parser.add_argument('-r', type=float, help="Threshold for which the value must be above/below", dest="threshold", required=True)
    parser.add_argument('-b', default="below=", help="Bin type", dest="bin_type")

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(0)

    args = parser.parse_args()

    ifile = verif.input.get_input(args.ifile)
    locations = ifile.locations
    locationids = [loc.id for loc in locations]
    leadtimes = ifile.leadtimes
    times = ifile.times
    lats = [loc.lat for loc in locations]
    lons = [loc.lon for loc in locations]
    elevs = [loc.elev for loc in locations]

    intervals = verif.util.get_intervals(args.bin_type, [args.threshold])
    if len(intervals) != 1:
        verif.util.error("Improper bin type '%s'" % args.bin_type)
    interval = intervals[0]

    fcst = copy.deepcopy(ifile.fcst)
    obs = copy.deepcopy(ifile.obs)

    fcst = calculate_window(fcst, args.threshold, interval, leadtimes)
    obs = calculate_window(obs, args.threshold, interval, leadtimes)

    file = netCDF4.Dataset(args.ofile, 'w', format="NETCDF4")
    file.createDimension("leadtime", len(ifile.leadtimes))
    file.createDimension("time", None)
    file.createDimension("location", len(ifile.locations))
    vTime = file.createVariable("time", "i4", ("time",))
    vOffset = file.createVariable("leadtime", "f4", ("leadtime",))
    vLocation = file.createVariable("location", "f8", ("location",))
    vLat = file.createVariable("lat", "f4", ("location",))
    vLon = file.createVariable("lon", "f4", ("location",))
    vElev = file.createVariable("altitude", "f4", ("location",))
    vfcst = file.createVariable("fcst", "f4", ("time", "leadtime", "location"))
    vobs = file.createVariable("obs", "f4", ("time", "leadtime", "location"))
    file.long_name = ifile.variable.name
    file.units = unit = ifile.variable.units.replace("$", "")
    file.Convensions = "verif_1.0.0"

    vobs[:] = obs
    vfcst[:] = fcst
    vTime[:] = times
    vOffset[:] = leadtimes
    vLocation[:] = locationids
    vLat[:] = lats
    vLon[:] = lons
    vElev[:] = elevs

    file.close()

if __name__ == '__main__':
    main()



import unittest
import numpy as np
import verif
import verif.input
import verif.data
import verif.location


class InputTextTest(unittest.TestCase):
    def test_get_leadtimes(self):
        input = verif.input.Text("verif/tests/files/example.txt")
        leadtimes = input.leadtimes
        self.assertEqual(4, leadtimes.shape[0])
        self.assertEqual(0, leadtimes[0])
        self.assertEqual(1, leadtimes[1])
        self.assertEqual(3, leadtimes[2])
        self.assertEqual(22, leadtimes[3])

    def test_get_times(self):
        input = verif.input.Text("verif/tests/files/example.txt")
        times = input.times
        self.assertEqual(1, times.shape[0])
        self.assertEqual(1325376000, times[0])

    def test_get_locations_no_id(self):
        input = verif.input.Text("verif/tests/files/example.txt")
        locations = input.locations
        # locations = np.sort(locations) # TODO does it have to be sorted?
        # Reset ids, to make it easier to verify what locations are available
        for loc in locations:
            loc.id = 0
        self.assertEqual(4, len(locations))
        self.assertTrue(verif.location.Location(0, 1, 1, 1) in locations)
        self.assertTrue(verif.location.Location(0, 0, 0, 1) in locations)
        self.assertTrue(verif.location.Location(0, 0, 0, 2) in locations)
        self.assertTrue(verif.location.Location(0, 2, 2, 1) in locations)

    def test_get_locations(self):
        input = verif.input.Text("verif/tests/files/file1.txt")
        locations = input.locations
        # locations = np.sort(locations) # TODO does it have to be sorted?
        self.assertEqual(2, len(locations))
        self.assertTrue(verif.location.Location(3, 50, 10, 12) in locations)
        self.assertTrue(verif.location.Location(41, 42, 23, 341) in locations)

    def test_get_locations_same_lat_lon(self):
        input1 = verif.input.Text("verif/tests/files/file1.txt")
        input2 = verif.input.Text("verif/tests/files/file1_same_lat_lon.txt")
        locations1 = input1.locations
        locations2 = input2.locations
        self.assertEqual(len(locations1), len(locations2))

        # Find the ordering of locations, such that obs, fcsts can be compared
        # Create a list of indicies that map location1 into location 2
        I = list()
        for i in range(0, len(locations1)):
            for j in range(0, len(locations2)):
                if locations1[i].id == locations2[j].id:
                    I += [j]

        self.assertEqual(len(locations1), len(I))
        # locations2 = np.sort(locations2) # TODO does it have to be sorted?
        self.assertEqual(2, len(locations2))
        self.assertTrue(verif.location.Location(3, 50, 10, 12) in locations2)
        self.assertTrue(verif.location.Location(41, 50, 10, 12) in locations2)

        np.testing.assert_array_equal(input1.obs[:, :, I], input2.obs)
        np.testing.assert_array_equal(input1.fcst[:, :, I], input2.fcst)
        np.testing.assert_array_equal(input1.leadtimes, input2.leadtimes)
        np.testing.assert_array_equal(input1.times, input2.times)

    def test_offset_leadtime(self):
        input1 = verif.input.Text("verif/tests/files/file1.txt")
        input2 = verif.input.Text("verif/tests/files/file1_leadtime.txt")
        np.testing.assert_array_equal(input1.obs, input2.obs)
        np.testing.assert_array_equal(input1.fcst, input2.fcst)
        np.testing.assert_array_equal(input1.leadtimes, input2.leadtimes)

    def test_conflictingLocations(self):
        data = verif.data.Data(
            inputs=[verif.input.Text("verif/tests/files/fileConflictingInfo.txt")]
        )

    def test_noId(self):
        data = verif.data.Data(
            inputs=[verif.input.Text("verif/tests/files/fileNoLocation.txt")]
        )

    def test_noElev(self):
        data = verif.data.Data(
            inputs=[verif.input.Text("verif/tests/files/fileNoElev.txt")]
        )
        self.assertEqual(2, len(data.locations))

    def test_noLat(self):
        data = verif.data.Data(
            inputs=[verif.input.Text("verif/tests/files/fileNoLat.txt")]
        )
        self.assertEqual(2, len(data.locations))

    def test_odd_header_names(self):
        # Check that names 'q', 'p', and 'e' can be read
        data = verif.data.Data(
            inputs=[verif.input.Text("verif/tests/files/file_odd_header_names.txt")]
        )

    def test_hour(self):
        # Check that the hour column works
        input = verif.input.Text("verif/tests/files/text_hour.txt")
        self.assertEqual((3, 2, 1), input.obs.shape)
        np.testing.assert_array_equal(input.leadtimes, [0, 2])
        ut = verif.util.date_to_unixtime(20180101)
        np.testing.assert_array_equal(input.times, [ut, ut + 12 * 3600, ut + 36 * 3600])
        self.assertTrue(np.isnan(input.obs[2, 0, 0]))
        self.assertEqual(9, input.obs[2, 1, 0])

    def test_get_scores(self):
        input = verif.input.Text("verif/tests/files/example.txt")
        obs = input.obs
        fcst = input.fcst
        self.assertEqual((1, 4, 4), obs.shape)
        self.assertEqual((1, 4, 4), fcst.shape)
        locations = input.locations
        self.assertEqual(4, len(locations))
        # Reset ids, to make it easier to verify what locations are available
        for loc in locations:
            loc.id = 0
        I0 = -1
        for i in range(0, len(locations)):
            if locations[i] == verif.location.Location(0, 1, 1, 1):
                I0 = i
            if locations[i] == verif.location.Location(0, 0, 0, 2):
                I1 = i
        self.assertEqual(12, obs[0, 0, I0])
        self.assertTrue(np.isnan(obs[0, 1, I0]))
        self.assertTrue(np.isnan(obs[0, 2, I0]))
        self.assertEqual(5, obs[0, 3, I0])
        self.assertEqual(13, fcst[0, 0, I0])
        self.assertTrue(np.isnan(fcst[0, 1, I0]))
        self.assertTrue(np.isnan(fcst[0, 2, I0]))
        self.assertEqual(4, fcst[0, 3, I0])

        for i in range(0, obs.shape[1]):
            self.assertTrue(np.isnan(obs[0, i, I1]))
        self.assertEqual(11, fcst[0, 0, I1])
        for i in range(1, obs.shape[1]):
            self.assertTrue(np.isnan(obs[0, i, I1]))

    def test_compatibility(self):
        input = verif.input.Text("verif/tests/files/file1_compatibility.txt")
        locations = input.locations
        # locations = np.sort(locations) # TODO does it have to be sorted?
        self.assertEqual(2, len(locations))
        self.assertTrue(verif.location.Location(3, 50, 10, 12) in locations)
        self.assertTrue(verif.location.Location(41, 42, 23, 341) in locations)
        np.testing.assert_array_equal(np.sort(input.leadtimes), [0, 6, 12])

    def test_order(self):
        # Check that the order of leadtimes in the file does not matter
        input1 = verif.input.Text("verif/tests/files/file_order1.txt")
        input2 = verif.input.Text("verif/tests/files/file_order2.txt")
        data = verif.data.Data([input1, input2])
        np.testing.assert_array_equal(
            np.sort(input1.leadtimes), np.sort(input2.leadtimes)
        )
        np.testing.assert_array_equal(np.sort(data.leadtimes), [0, 6, 12])
        s1 = data.get_scores(verif.field.Fcst(), 0)
        s2 = data.get_scores(verif.field.Fcst(), 1)
        np.testing.assert_array_equal(s1, s2)

    def test_attributes(self):
        input = verif.input.Text("verif/tests/files/file1_x0_x1.txt")
        data = verif.data.Data([input])
        self.assertEqual(0, data.variable.x0)
        self.assertEqual(10, data.variable.x1)
        self.assertEqual("Weird variable", data.variable.name)
        self.assertEqual("Some units", data.variable.units)

    def test_missing_header(self):
        with self.assertRaises(SystemExit) as context:
            input = verif.input.Text("verif/tests/files/file1_no_header.txt")

    def test_missing_values(self):
        # Check NA, na, nan are detected
        input = verif.input.Text("verif/tests/files/file_missing_values.txt")
        data = verif.data.Data([input])
        obs = data.get_scores(verif.field.Obs(), 0)
        fcst = data.get_scores(verif.field.Fcst(), 0)
        self.assertEqual(len(data.locations), 1)

        self.assertEqual(obs[0, 0, 0], 3)
        self.assertEqual(fcst[0, 0, 0], 6)

        self.assertTrue(np.isnan(obs[0, 2, 0]))
        self.assertEqual(fcst[0, 2, 0], 7)

        self.assertEqual(obs[0, 3, 0], 3)
        self.assertTrue(np.isnan(fcst[0, 3, 0]))

        self.assertTrue(np.isnan(obs[0, 4, 0]))
        self.assertTrue(np.isnan(fcst[0, 4, 0]))

    def test_ensemble(self):
        input = verif.input.Text("verif/tests/files/file1_ens.txt")

        self.assertEqual(input.ensemble.shape, (3, 3, 2, 3))
        Iloc = [l.id for l in input.locations].index(3)
        np.testing.assert_array_almost_equal(input.ensemble[0, 0, Iloc, :], [3, 4, 5])

        data = verif.data.Data([input])
        ens = data.get_scores(verif.field.Ensemble(), 0)

        self.assertEqual(ens.shape, (3, 3, 2, 3))
        self.assertEqual(ens[0, 0, 0, 0], 3)

    def test_ensemble_member(self):
        input = verif.input.Text("verif/tests/files/file1_ens.txt")

        self.assertEqual(input.ensemble.shape, (3, 3, 2, 3))
        Iloc = [l.id for l in input.locations].index(3)
        np.testing.assert_array_almost_equal(input.ensemble[0, 0, Iloc, :], [3, 4, 5])

        data = verif.data.Data([input])
        ens = data.get_scores(verif.field.EnsembleMember(0), 0)

        self.assertEqual(ens.shape, (3, 3, 2))
        self.assertEqual(ens[0, 0, 0], 3)


if __name__ == "__main__":
    unittest.main()



import unittest
import verif.aggregator
import numpy as np
import numbers


class TestAggregator(unittest.TestCase):
    def test_simple(self):
        agg = verif.aggregator.Mean()
        self.assertEqual(2, agg(np.array([1, 2, 3])))

    def test_name(self):
        mean = verif.aggregator.Mean()
        self.assertEqual("mean", mean.name())
        mean2 = verif.aggregator.get("mean")
        self.assertEqual("mean", mean2.name())
        self.assertEqual(mean, mean2)

    def test_equal(self):
        self.assertEqual(verif.aggregator.Mean(), verif.aggregator.Mean())
        self.assertTrue(verif.aggregator.Mean() == verif.aggregator.Mean())
        self.assertFalse(verif.aggregator.Mean() != verif.aggregator.Mean())

        self.assertEqual(verif.aggregator.Quantile(0.2), verif.aggregator.Quantile(0.2))
        self.assertTrue(
            verif.aggregator.Quantile(0.2) == verif.aggregator.Quantile(0.2)
        )
        self.assertFalse(
            verif.aggregator.Quantile(0.2) != verif.aggregator.Quantile(0.2)
        )

    def test_unequal(self):
        self.assertFalse(verif.aggregator.Mean() == verif.aggregator.Absmean())
        self.assertFalse(verif.aggregator.Mean() == verif.aggregator.Quantile(0.2))
        self.assertTrue(verif.aggregator.Mean() != verif.aggregator.Absmean())
        self.assertTrue(verif.aggregator.Mean() != verif.aggregator.Quantile(0.2))

        self.assertFalse(
            verif.aggregator.Quantile(0.2) == verif.aggregator.Quantile(0.3)
        )
        self.assertTrue(
            verif.aggregator.Quantile(0.2) != verif.aggregator.Quantile(0.3)
        )

    def test_call(self):
        aggregators = [verif.aggregator.Mean(), verif.aggregator.Min()]
        aggregators = list()
        for a in verif.aggregator.get_all():
            if a == verif.aggregator.Quantile:
                aggregators += [a(0.9)]
            else:
                aggregators += [a()]

        array = np.array([[0, 1, 2], [10, 11.0, 12]])
        for aggregator in aggregators:
            value = aggregator(array)
            self.assertTrue(isinstance(value, numbers.Number))
            value = aggregator(array, axis=0)
            self.assertEqual(value.shape, (3,))
            value = aggregator(array, axis=1)
            self.assertEqual(value.shape, (2,))


if __name__ == "__main__":
    unittest.main()



import unittest
import verif.driver
import os
import numpy as np
import tempfile

np.seterr("raise")


all_axes = [
    "time",
    "leadtime",
    "timeofday",
    "dayofyear",
    "monthofyear",
    "day",
    "week",
    "month",
    "year",
    "leadtimeday",
    "location",
    "lat",
    "lon",
    "elev",
]


class IntegrationTest(unittest.TestCase):
    """
    These tests run verif on the command-line, but do not test the validity of the
    output graphics, only that they do or do not create errors.
    """

    @staticmethod
    def run_command(command):
        """Runs a verif command line"""
        argv = command.split()
        verif.driver.run(argv)

    @staticmethod
    def remove(file):
        """Removes a file"""
        os.remove(file)

    @staticmethod
    def file_size(filename):
        """Returns the number of bytes of a file"""
        statinfo = os.stat(filename)
        return statinfo.st_size

    @staticmethod
    def is_valid_file(filename, min_size=3000):
        """Checks if a file is larger in size than min_size bytes"""
        return IntegrationTest.file_size(filename) > min_size

    def run_with_image(self, command):
        """
        Runs the verif command and appends -f <somefile>.png so that it will write output
        to a temporary png file. Removes the file afterwards.
        """
        fd, imageFile = tempfile.mkstemp(suffix=".png")
        command = command + " -f " + imageFile
        self.run_command(command)
        self.assertTrue(self.is_valid_file(imageFile), 3000)
        os.close(fd)
        self.remove(imageFile)

    def run_with_text(self, command):
        """
        Runs the verif command and appends -f <somefile>.txt so that it will write output
        to a temporary txt file. Removes the file afterwards.
        """
        fd, textFile = tempfile.mkstemp(suffix=".txt")
        command = command + " -f " + textFile
        self.run_command(command)
        self.assertTrue(self.is_valid_file(textFile, 10))
        os.close(fd)
        self.remove(textFile)

    def test_valid(self):
        self.run_command("verif")
        self.run_command("verif --help")
        self.run_command("verif --version")
        self.run_command("verif examples/raw.txt examples/kf.txt --list-thresholds")
        self.run_command("verif examples/raw.txt examples/kf.txt --list-quantiles")
        self.run_command("verif examples/raw.txt examples/kf.txt --list-times")
        self.run_command("verif examples/raw.txt examples/kf.txt --list-dates")
        self.run_command(
            "verif examples/raw.txt examples/kf.txt --list-thresholds --list-quantiles --list-times"
        )

    def test_invalid(self):
        with self.assertRaises(SystemExit):
            self.run_with_image("verif --list-thresholds")
        with self.assertRaises(SystemExit):
            self.run_with_image("verif --list-quantiles")
        with self.assertRaises(SystemExit):
            self.run_with_image("verif --list-times")

    def test_README(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m ets")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m taylor")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m error")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m reliability -r 0"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m pithist")

    def test_option_b(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m ets -b below")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m ets -b within")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m ets -b above")

    def test_option_c(self):
        self.run_with_image("verif examples/raw.txt -c examples/kf.txt -m ets")

    def test_option_leg(self):
        self.run_with_image("verif -leg 1,2 examples/raw.txt examples/kf.txt -m ets")
        self.run_with_image(
            "verif -leg 1,2 examples/raw.txt examples/kf.txt -m ets -x no"
        )
        self.run_with_image(
            "verif -leg 1dqwoijdioqwjdoiqjwdoijiqow,2dqwoijdioqwjdoiqjwdoijiqow examples/raw.txt examples/kf.txt -m ets"
        )
        with self.assertRaises(SystemExit):
            self.run_with_image("verif -leg 1 examples/raw.txt examples/kf.txt -m ets")

    def test_option_ct(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -agg min")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -agg mean")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -agg median")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -agg max")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -agg std")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -agg range")

    def test_standard_option_x(self):
        for axis in all_axes:
            self.run_with_image(
                "verif examples/raw.txt examples/kf.txt -m mae -x %s" % axis
            )

    def test_obsfcst_option_x(self):
        for axis in all_axes:
            self.run_with_image(
                "verif examples/raw.txt examples/kf.txt -m obsfcst -x %s" % axis
            )

    def test_obsfcst_option_acc(self):
        for axis in all_axes:
            self.run_with_image(
                "verif examples/raw.txt examples/kf.txt -m obsfcst -x %s -acc" % axis
            )

    def test_pithist(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m pithistdev")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m pithistslope")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m pithistshape")

    def test_obs_subset(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -r 10")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -x threshold"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -r 0,2,10 -x threshold -b within"
        )

    def test_annotate(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -a")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -a -x location"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -a -type map"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m obsfcst -a")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m obsfcst -a -x location"
        )

        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -a -type map -af score,location"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -a -type map -afs 4"
        )

    def test_plotting_options(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -aspect 0.1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -aspect 2.1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -bottom 0.1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -bottom 0.5")
        # -clim and -cmap are tested with -type map
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -dpi 50")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -dpi 300")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -fs 10,2")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -gc red")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -gs -")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -gw 2")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -labfs 0")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -labfs 11")
        # -lc tests are in separate functions
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -left 0.8")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -legfs 0")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -legfs 10")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -legloc right"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -legloc lower_left"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -ls - -ma ,o"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -ls None -ma *"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -ls -,-, -ma ,s,:"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -lw 0")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -lw 1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -lw 1.3")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -lw 2")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -ms 0")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -ms 1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -ms 1.3")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -ms 2")
        # For some reason this fails without -left 0.1, although it works fine when verif is
        # invoked on the command line:
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -left 0.1 -right 0.8"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -tickfs 0")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -tickfs 10")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -title title"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -title title_with_underscores"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -titlefs 0")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -titlefs 10")
        # Same as for -right above
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -bottom 0.1 -top 0.4"
        )
        # -type is tested separately
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -xlabel test"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -xlim 0,1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -xrot 90")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -xticks 0:4")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -xticks 0:3 -xticklabels 0,test,1,2"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -xticklabels 0,test,1"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -xticklabels ''"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -ylabel test"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -ylim 0,1")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -yrot 90")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -yticks 0:4")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -yticks 0:3 -yticklabels 0,test,1,2"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -yticklabels 0,test,1"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -yticklabels ''"
        )

    def test_leadtime_aggregation(self):
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -T 1 -Tx leadtime"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -T 2 -Tx leadtime"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -T 2 -Tx time"
        )
        with self.assertRaises(SystemExit):
            self.run_with_image(
                "verif examples/raw.txt examples/kf.txt -m mae -T 1 -Tx location"
            )

    def test_against(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m against")
        # Ensure at least 3 files to test the subplots
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt examples/raw.txt -m against"
        )

    def test_impact(self):
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -type impact"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m corr -type impact -ms 6 -r 0:0.1:1"
        )

    def test_mapimpact(self):
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -type mapimpact -legfs 0"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m ets -type mapimpact -r 1 -legfs 0"
        )

    def test_fss(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m fss -r 5")
        self.run_with_image("verif verif/tests/files/file3.txt -m fss -r 5")
        self.run_with_image("verif verif/tests/files/file3.txt -m fss -r 100")
        self.run_with_image(
            "verif verif/tests/files/file3.txt -m fss -r 0.1 -x leadtime"
        )
        with self.assertRaises(SystemExit):
            self.run_with_image(
                "verif verif/tests/files/file3.txt -m fss -r 0.1 -x time"
            )

    def test_taylor(self):
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m taylor -xlim 0,2"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m taylor -xlim 0,0.2"
        )

    def test_roc(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m droc -r 0")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m droc -r 0 -simple"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m droc -r 0 -xlog")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m droc -r 0 -ylog")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m droc -r 0 -xlog -ylog"
        )

    def test_obsleg(self):
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m obsfcst -obsleg Test -leg 1,2"
        )

    def test_discrimination(self):
        self.run_with_image("verif examples/raw.txt -m discrimination -r 0")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m discrimination -r 0"
        )

    def test_scatter(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m scatter")

    def test_auto(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m autocorr")
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m autocov")

    def test_roc(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m roc -r 0")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m roc -r 0 -b below"
        )

    def test_autocorr(self):
        self.run_with_image("verif examples/raw.txt -m autocorr")
        self.run_with_image("verif examples/raw.txt -m autocorr -x location")
        self.run_with_image("verif examples/raw.txt -m autocorr -x lat")
        self.run_with_image("verif examples/raw.txt -m autocorr -x lon")
        self.run_with_image("verif examples/raw.txt -m autocorr -x elev")
        self.run_with_image("verif examples/raw.txt -m autocorr -x time")
        self.run_with_image("verif examples/raw.txt -m autocorr -x leadtime")
        self.run_with_image("verif examples/raw.txt -m autocorr -r 0:100:1000")
        self.run_with_image(
            "verif examples/raw.txt -m autocorr -r 0:100:1000 -xlim 0,100"
        )

    def test_cond(self):
        self.run_with_image("verif examples/raw.txt -m cond -r 1:10")

    def test_qq(self):
        self.run_with_image("verif examples/raw.txt -m qq -sp")
        self.run_with_image("verif examples/raw.txt -m qq -x location")

    def test_spreadskill(self):
        self.run_with_image("verif verif/tests/files/file1_ens.txt -m spreadskill -sp")

    def test_timeseries(self):
        self.run_with_image("verif examples/raw.txt -m timeseries")
        self.run_with_image("verif examples/raw.txt -m meteo")

    def test_murphy(self):
        self.run_with_image("verif examples/raw.txt -m murphy -r 0")

    def test_bsdecomp(self):
        self.run_with_image("verif examples/raw.txt -m bsdecomp -r 0")

    def test_config(self):
        self.run_with_image(
            "verif examples/raw.txt --config verif/tests/files/config1.txt"
        )
        self.run_with_image(
            "verif examples/raw.txt -m mae --config verif/tests/files/configEmpty.txt"
        )

    def test_other_fields(self):
        self.run_with_image("verif verif/tests/files/file1_crps.txt -m ensemble_crps")
        self.run_with_image(
            "verif verif/tests/files/file1_crps.txt -m ensemble_crps -x time"
        )
        self.run_with_image(
            "verif verif/tests/files/file1_crps.txt -m ensemble_crps -agg median"
        )

    def test_crps(self):
        self.run_with_image("verif verif/tests/files/file1_ens.txt -m fcrps")
        self.run_with_image("verif verif/tests/files/file1_ens.txt -m fcrps -x time")
        self.run_with_image("verif verif/tests/files/file1_ens.txt -m fcrps -agg median")

    def test_map_type(self):
        pass

    def test_type(self):
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -type rank")
        # These cause a FutureWarning in mpl, but not much we can do about that
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -type map -clim 0,11"
        )
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -type map -cmap RdBu"
        )
        self.run_with_image("verif examples/raw.txt examples/kf.txt -m mae -type map")
        self.run_with_image(
            "verif examples/raw.txt examples/kf.txt -m mae -type maprank"
        )

    def test_text(self):
        self.run_with_text("verif examples/raw.txt examples/kf.txt -m mae -type text")
        self.run_with_text(
            "verif examples/raw.txt examples/kf.txt -m ets -r 0,1,2 -x threshold -type text"
        )
        self.run_with_text(
            "verif examples/raw.txt examples/kf.txt -m mae -x fcst -type text"
        )
        self.run_with_text(
            "verif examples/raw.txt examples/kf.txt -m mae -x obs -type text"
        )

    def test_csv(self):
        self.run_with_text("verif examples/raw.txt examples/kf.txt -m mae -type csv")
        self.run_with_text(
            "verif examples/raw.txt examples/kf.txt -m ets -r 0,1,2 -x threshold -type csv"
        )
        self.run_with_text(
            "verif examples/raw.txt examples/kf.txt -m mae -x fcst -type csv"
        )
        self.run_with_text(
            "verif examples/raw.txt examples/kf.txt -m mae -x obs -type csv"
        )

    def test_freq(self):
        self.run_with_image("verif verif/tests/files/file1.txt -m freq")
        # Check that no error occurs, even though fcst or obs is not available
        self.run_with_image("verif verif/tests/files/file1_no_obs.txt -m freq")
        self.run_with_image("verif verif/tests/files/file1_no_fcst.txt -m freq")

    def test_option_lc(self):
        for lc in ("g,r", "g", "g,r,b", "0,0.5,0.9", "[0,0,1],0.5,g"):
            self.run_with_image(
                "verif examples/raw.txt examples/kf.txt -m mae -lc %s" % lc
            )

    def test_boolean_options(self):
        for opt in (
            "acc",
            "nogrid",
            "nomargin",
            "hist",
            "sort",
            "sp",
            "simple",
            "xlog",
            "ylog",
        ):
            self.run_with_image(
                "verif examples/raw.txt examples/kf.txt -m obs -%s" % opt
            )

    def test_invalidMetric(self):
        with self.assertRaises(SystemExit):
            self.run_with_image("verif examples/T_raw_0.nc -m maeq")

    def test_invalidFile(self):
        with self.assertRaises(SystemExit):
            self.run_with_image("verif examples/T_raw_1.nc -m mae")


if __name__ == "__main__":
    unittest.main()



import argparse
import netCDF4
import numpy as np
import sys


import verif.input
import verif.data
import verif.field


def main():
    parser = argparse.ArgumentParser(prog="text2verif", description="Convert between Verif text and NetCDF files")
    parser.add_argument('ifile', type=str, help="Verif text file (input)")
    parser.add_argument('ofile', type=str, help="Verif NetCDF file (output)")
    parser.add_argument('--debug', help='Print debug information', action="store_true")

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(0)

    args = parser.parse_args()

    input = verif.input.get_input(args.ifile)
    times = input.times
    leadtimes = input.leadtimes
    locations = input.locations
    variable = input.variable

    output = netCDF4.Dataset(args.ofile, 'w')
    output.createDimension("time", None)
    output.createDimension("leadtime", len(leadtimes))
    output.createDimension("location", len(locations))
    thresholds = input.thresholds
    quantiles = input.quantiles
    if len(thresholds) > 0:
        if args.debug:
            print("Adding %d thresholds" % len(thresholds))
        output.createDimension("threshold", len(thresholds))
        output.createVariable("threshold", "f4", ["threshold"])
        output["threshold"][:] = thresholds
        output.createVariable("cdf", "f4", ("time", "leadtime", "location", "threshold"))
        output.variables["cdf"][:] = input.threshold_scores
    if len(quantiles) > 0:
        if args.debug:
            print("Adding %d quantiles" % len(quantiles))
        output.createDimension("quantile", len(quantiles))
        output.createVariable("quantile", "f4", ["quantile"])
        output["quantile"][:] = quantiles
        output.createVariable("x", "f4", ("time", "leadtime", "location", "quantile"))
        output.variables["x"][:] = input.quantile_scores

    vTime = output.createVariable("time", "f8", ("time",))
    vOffset = output.createVariable("leadtime", "f4", ("leadtime",))
    vLocation = output.createVariable("location", "i4", ("location",))
    vLat = output.createVariable("lat", "f4", ("location",))
    vLon = output.createVariable("lon", "f4", ("location",))
    vElev = output.createVariable("altitude", "f4", ("location",))
    vfcst = output.createVariable("fcst", "f4", ("time", "leadtime", "location"))
    vobs = output.createVariable("obs", "f4", ("time", "leadtime", "location"))

    # Create nonstandard fields
    standard = [verif.field.Obs(), verif.field.Fcst()]
    fields = [field for field in input.get_fields() if field not in standard]
    for field in fields:
        name = field.name()
        if field.__class__ == verif.field.Other:
            if args.debug:
                print("Adding non-standard score '%s'" % name)
            output.createVariable(name, "f4", ("time", "leadtime", "location"))
            output.variables[name][:] = input.other_score(field.name())[:]

    output.standard_name = variable.name
    output.units = unit = variable.units.replace("$", "")

    vobs[:] = input.obs
    vfcst[:] = input.fcst
    vTime[:] = input.times
    vOffset[:] = input.leadtimes
    vLocation[:] = [s.id for s in locations]
    vLat[:] = [s.lat for s in locations]
    vLon[:] = [s.lon for s in locations]
    vElev[:] = [s.elev for s in locations]
    output.Conventions = "verif_1.0.0"
    output.close()


if __name__ == '__main__':
    main()



from builtins import str

import sys
import os
import verif.axis
import verif.data
import verif.input
import verif.metric
import verif.metric_type
import verif.output
import verif.util
import verif.version
import textwrap
import numpy as np


def run(argv):
    ############
    # Defaults #
    ############
    ifiles = list()
    ofile = None
    metric = None
    locations = None
    locations_x = None
    lat_range = None
    lon_range = None
    elev_range = None
    obs_range = None
    thresholds = None
    quantiles = None
    clim_file = None
    clim_type = "subtract"
    leg = None
    ylabel = None
    xlabel = None
    clabel = None
    title = None
    dates = None
    tods = None
    times = None
    leadtimes = None
    axis = None
    aspect = None
    figsize = None
    dpi = 100
    no_margin = False
    bin_type = None
    simple = None
    marker_sizes = None
    line_widths = None
    line_colors = None
    line_styles = None
    grid_style = None
    grid_color = None
    grid_width = None
    markers = None
    tick_font_size = None
    lab_font_size = None
    leg_font_size = None
    title_font_size = None
    annotation_font_size = None
    leg_loc = None
    plot_type = "plot"
    grid = True
    xrot = None
    yrot = None
    bottom_padding = None
    top_padding = None
    right_padding = None
    left_padding = None
    Pad = None
    show_perfect = None
    aggregator_name = None
    do_hist = False
    do_sort = False
    do_acc = False
    xlim = None
    ylim = None
    clim = None
    annotate = False
    annotation_format = None
    xticks = None
    xticklabels = None
    yticks = None
    yticklabels = None
    version = None
    list_thresholds = False
    list_quantiles = False
    list_locations = False
    list_times = False
    list_dates = False
    map_type = None
    xlog = False
    ylog = False
    cmap = None
    obs_leg = None
    obs_field = verif.field.Obs()
    fcst_field = verif.field.Fcst()
    dim_agg_axis = verif.axis.Leadtime()
    dim_agg_method = verif.aggregator.Mean()
    dim_agg_length = None
    show_help = False

    # Parse config files
    i = 1
    extra = []
    while(i < len(argv)):
        arg = argv[i]
        if arg == "--config":
            if i == len(argv) - 1:
                verif.util.error("Missing filename after --config")
            i = i + 1
            filename = argv[i]
            try:
                fid = open(filename, 'r')
                for line in fid:
                    extra += line.split()
            except:
                if not os.path.isfile(filename):
                    verif.util.error("Could not read %s" % filename)
        i = i + 1

    argv = argv + extra

    # Read command line arguments
    i = 1
    while(i < len(argv)):
        arg = argv[i]
        if arg[0] == '-':
            # Process option
            if arg == "-nomargin":
                no_margin = True
            elif arg == "--version":
                version = True
            elif arg == "--list-thresholds":
                list_thresholds = True
            elif arg == "--list-quantiles":
                list_quantiles = True
            elif arg == "--list-locations":
                list_locations = True
            elif arg == "--list-times":
                list_times = True
            elif arg == "--list-dates":
                list_dates = True
            elif arg == "-sp":
                show_perfect = True
            elif arg == "-hist":
                do_hist = True
            elif arg == "-acc":
                do_acc = True
            elif arg == "-sort":
                do_sort = True
            elif arg == "-simple":
                simple = True
            elif arg == "-xlog":
                xlog = True
            elif arg == "-ylog":
                ylog = True
            elif arg == "-a":
                annotate = True
            elif arg == "-nogrid":
                grid = False
            elif arg == "--help":
                show_help = True
            else:
                if len(argv) <= i + 1:
                    verif.util.error("Missing value after %s" % argv[i])
                arg_next = argv[i + 1]
                if arg == "-f":
                    ofile = arg_next
                elif arg == "-l":
                    locations = verif.util.parse_numbers(arg_next)
                elif arg == "-lx":
                    locations_x = verif.util.parse_numbers(arg_next)
                elif arg == "-latrange":
                    lat_range = verif.util.parse_numbers(arg_next)
                elif arg == "-lonrange":
                    lon_range = verif.util.parse_numbers(arg_next)
                elif arg == "-elevrange":
                    elev_range = verif.util.parse_numbers(arg_next)
                elif arg == "-obsrange":
                    obs_range = verif.util.parse_numbers(arg_next)
                elif arg == "-x":
                    axisname = arg_next
                    axis = verif.axis.get(axisname)
                elif arg == "-o":
                    leadtimes = verif.util.parse_numbers(arg_next)
                elif arg == "-leg":
                    leg = verif.util.parse_label(arg_next)
                elif arg == "-ylabel":
                    ylabel = verif.util.parse_label(arg_next)
                elif arg == "-xlabel":
                    xlabel = verif.util.parse_label(arg_next)
                elif arg == "-clabel":
                    clabel = verif.util.parse_label(arg_next)
                elif arg == "-title":
                    title = verif.util.parse_label(arg_next)
                elif arg == "-b":
                    bin_type = arg_next
                elif arg == "-type":
                    plot_type = arg_next
                elif arg == "-fs":
                    figsize = arg_next
                elif arg == "-dpi":
                    dpi = int(arg_next)
                elif arg == "-d":
                    dates = verif.util.parse_numbers(arg_next, True)
                elif arg == "-tod":
                    tods = [int(tod) for tod in verif.util.parse_numbers(arg_next)]
                elif arg == "-t":
                    times = verif.util.parse_numbers(arg_next)
                elif arg == "-c":
                    clim_file = verif.input.get_input(arg_next)
                    clim_type = "subtract"
                elif arg == "-C":
                    clim_file = verif.input.get_input(arg_next)
                    clim_type = "divide"
                elif arg == "-xlim":
                    xlim = verif.util.parse_numbers(arg_next)
                elif arg == "-ylim":
                    ylim = verif.util.parse_numbers(arg_next)
                elif arg == "-clim":
                    clim = verif.util.parse_numbers(arg_next)
                elif arg == "-xticks":
                    xticks = verif.util.parse_numbers(arg_next)
                elif arg == "-xticklabels":
                    xticklabels = verif.util.parse_label(arg_next).split(',')
                elif arg == "-yticks":
                    yticks = verif.util.parse_numbers(arg_next)
                elif arg == "-yticklabels":
                    yticklabels = verif.util.parse_label(arg_next).split(',')
                elif arg == "-agg":
                    aggregator_name = arg_next
                elif arg == "-aspect":
                    aspect = float(arg_next)
                elif arg == "-r":
                    thresholds = np.array(verif.util.parse_numbers(arg_next))
                elif arg == "-q":
                    quantiles = np.array(verif.util.parse_numbers(arg_next))
                    if np.min(quantiles) < 0 or np.max(quantiles) > 1:
                        verif.util.error("Quantiles must be between 0 and 1 inclusive")
                elif arg == "-ms":
                    marker_sizes = verif.util.parse_ints(arg_next)
                elif arg == "-lw":
                    line_widths = verif.util.parse_numbers(arg_next)
                elif arg == "-lc":
                    line_colors = verif.util.parse_colors(arg_next)
                elif arg == "-ls":
                    line_styles = arg_next.split(',')
                elif arg == "-gs":
                    grid_style = arg_next
                elif arg == "-gc":
                    grid_color = arg_next
                elif arg == "-gw":
                    grid_width = arg_next
                elif arg == "-ma":
                    markers = arg_next.split(',')
                elif arg == "-tickfs":
                    tick_font_size = float(arg_next)
                elif arg == "-labfs":
                    lab_font_size = float(arg_next)
                elif arg == "-legfs":
                    leg_font_size = float(arg_next)
                elif arg == "-legloc":
                    leg_loc = arg_next.replace('_', ' ')
                elif arg == "-xrot":
                    xrot = float(arg_next)
                elif arg == "-yrot":
                    yrot = float(arg_next)
                elif arg == "-bottom":
                    bottom_padding = float(arg_next)
                elif arg == "-top":
                    top_padding = float(arg_next)
                elif arg == "-right":
                    right_padding = float(arg_next)
                elif arg == "-left":
                    left_padding = float(arg_next)
                elif arg == "-pad":
                    Pad = arg_next
                elif arg == "-titlefs":
                    title_font_size = float(arg_next)
                elif arg == "-cmap":
                    cmap = arg_next
                elif arg == "-maptype":
                    map_type = arg_next
                    if map_type not in verif.output.allowedMapTypes:
                        verif.util.error("Unknown maptype '%s'" % map_type)
                elif arg == "-obs":
                    obs_field = verif.field.get(arg_next)
                elif arg == "-fcst":
                    fcst_field = verif.field.get(arg_next)
                elif arg == "-obsleg":
                    obs_leg = arg_next
                elif arg == "-m":
                    metric = arg_next
                elif arg == "-T":
                    dim_agg_length = int(arg_next)
                elif arg == "-Tagg":
                    dim_agg_method = verif.aggregator.get(arg_next)
                elif arg == "-Tx":
                    axisname = arg_next
                    dim_agg_axis = verif.axis.get(axisname)
                elif arg == "-af":
                    annotation_format = arg_next.split(',')
                elif arg == "-afs":
                    annotation_font_size = float(arg_next)
                elif arg == "--config":
                    pass
                else:
                    verif.util.error("Flag '" + argv[i] + "' not recognized")
                i = i + 1
        else:
            ifiles.append(argv[i])
        i = i + 1

    if version:
        print("Version: " + verif.version.__version__)
        return

    # Deal with legend entries
    if leg is not None:
        leg = leg.split(',')
        for i in range(0, len(leg)):
            leg[i] = leg[i].replace('_', ' ')

    if title is not None:
        title = title.replace('_', ' ')

    if lat_range is not None and len(lat_range) != 2:
        verif.util.error("-latrange <values> must have exactly 2 values")

    if lon_range is not None and len(lon_range) != 2:
        verif.util.error("-lonrange <values> must have exactly 2 values")

    if elev_range is not None and len(elev_range) != 2:
        verif.util.error("-elevrange <values> must have exactly 2 values")

    if obs_range is not None and len(obs_range) != 2:
        verif.util.error("-obsrange <values> must have exactly 2 values")

    if dim_agg_length is not None and dim_agg_length <= 0:
        verif.util.error("-T <value> must be greater than 0")

    if len(ifiles) > 0:
        inputs = [verif.input.get_input(filename) for filename in ifiles]
        data = verif.data.Data(inputs, clim=clim_file, clim_type=clim_type,
              times=times, dates=dates, tods=tods, leadtimes=leadtimes, locations=locations,
              locations_x=locations_x,
              lat_range=lat_range, lon_range=lon_range, elev_range=elev_range,
              obs_range=obs_range, legend=leg, obs_field=obs_field, fcst_field=fcst_field,
              dim_agg_length=dim_agg_length,
              dim_agg_axis=dim_agg_axis,
              dim_agg_method=dim_agg_method)
    else:
        data = None

    if list_thresholds or list_quantiles or list_locations or list_times or list_dates:
        if len(ifiles) == 0:
            verif.util.error("Files are required in order to list thresholds, quantiles, or times")
        if list_thresholds:
            print("Thresholds:", end=' ')
            for threshold in data.thresholds:
                print("%g" % threshold, end=' ')
            print("")
        if list_quantiles:
            print("Quantiles:", end=' ')
            for quantile in data.quantiles:
                print("%g" % quantile, end=' ')
            print("")
        if list_locations:
            print("    id     lat     lon    elev")
            for location in data.locations:
                print("%6d %7.2f %7.2f %7.1f" % (location.id, location.lat,
                      location.lon, location.elev))
            print("")
        if list_times:
            for time in data.times:
                print("%d" % time)
            print("")
        if list_dates:
            for time in data.times:
                date = verif.util.unixtime_to_date(time)
                diff = time % 86400
                hour = diff / 3600
                minute = (diff % 3600)/60
                second = diff % 60
                print("%d %02d:%02d:%02d" % (date, hour, minute, second))
            print("")
        return
    elif len(ifiles) == 0 and metric is not None:
        m = verif.metric.get(metric)
        if m is not None:
            print(m.help())
        else:
            m = verif.output.get(metric)
            if m is not None:
                print(m.help())
        return
    elif len(argv) == 1 or len(ifiles) == 0 or metric is None or show_help:
        print(show_description(data))
        return

    if figsize is not None:
        figsize = figsize.split(',')
        if len(figsize) != 2:
            print("-fs figsize must be in the form: width,height")
            sys.exit(1)

    m = None

    # Handle special plots
    if metric == "pithist":
        pl = verif.output.PitHist()
    elif metric == "rankhist":
        pl = verif.output.RankHist()
    elif metric == "obsfcst":
        pl = verif.output.ObsFcst()
    elif metric == "timeseries":
        pl = verif.output.TimeSeries()
    elif metric == "meteo":
        pl = verif.output.Meteo()
    elif metric == "qq":
        pl = verif.output.QQ()
    elif metric == "autocorr":
        pl = verif.output.Auto("corr")
    elif metric == "autocov":
        pl = verif.output.Auto("cov")
    elif metric == "fss":
        pl = verif.output.Fss()
    elif metric == "cond":
        pl = verif.output.Cond()
    elif metric == "against":
        pl = verif.output.Against()
    elif metric == "scatter":
        pl = verif.output.Scatter()
    elif metric == "change":
        pl = verif.output.Change()
    elif metric == "spreadskill":
        pl = verif.output.SpreadSkill()
    elif metric == "taylor":
        pl = verif.output.Taylor()
    elif metric == "error":
        pl = verif.output.Error()
    elif metric == "freq":
        pl = verif.output.Freq()
    elif metric == "roc":
        pl = verif.output.Roc()
    elif metric == "droc":
        pl = verif.output.DRoc()
    elif metric == "droc0":
        pl = verif.output.DRoc0()
    elif metric == "reliability":
        pl = verif.output.Reliability()
    elif metric == "discrimination":
        pl = verif.output.Discrimination()
    elif metric == "performance":
        pl = verif.output.Performance()
    elif metric == "invreliability":
        pl = verif.output.InvReliability()
    elif metric == "murphy":
        pl = verif.output.Murphy()
    elif metric == "bsdecomp":
        pl = verif.output.BsDecomp()
    elif metric == "igncontrib":
        pl = verif.output.IgnContrib()
    elif metric == "economicvalue":
        pl = verif.output.EconomicValue()
    elif metric == "marginal":
        pl = verif.output.Marginal()
    else:
        # Standard plots
        # Attempt at automating
        m = verif.metric.get(metric)
        if m is None:
            m = verif.metric.FromField(verif.field.Other(metric))

        if aggregator_name is not None:
            if not m.supports_aggregator:
                verif.util.warning("-m %s does not support -agg" % metric)
            m.aggregator = verif.aggregator.get(aggregator_name)

        # Output type
        if plot_type in ["plot", "text", "csv", "map", "maprank", "rank", "impact", "mapimpact"]:
            if do_sort:
                field = verif.field.get(metric)
                pl = verif.output.Sort(field)
            elif do_hist:
                field = verif.field.get(metric)
                pl = verif.output.Hist(field)
            else:
                pl = verif.output.Standard(m)
        else:
            verif.util.error("Type not understood")

    # Rest dimension of '-x' is not allowed
    if axis is not None and not pl.supports_x:
        verif.util.warning("'-m %s'" % metric + " does not support '-x'. Ignoring it.")
        axis = None

    # Reset dimension if 'threshold' is not allowed
    if axis == verif.axis.Threshold() and ((not pl.supports_threshold) or (m is not None and not m.supports_threshold)):
        verif.util.warning("'-m %s'" % metric + " does not support '-x threshold'. Ignoring it.")
        thresholds = None
        axis = None
    # Reset dimension if 'obs' or 'fcst' is not allowed
    if axis in [verif.axis.Obs(), verif.axis.Fcst()] and ((not pl.supports_field) or (m is not None and not m.supports_field)):
        verif.util.warning("'-m %s'" % metric + " does not support '-x %s'. Ignoring it." % axis.name().lower())
        thresholds = None
        axis = None

    # Create thresholds if needed
    if thresholds is None:
        ttype = None
        if plot_type == "impact":
            ttype = "deterministic"
        elif pl.require_threshold_type == "deterministic":
            ttype = "deterministic"
        elif pl.require_threshold_type == "threshold":
            ttype = "threshold"
        elif m is not None:
            if m.require_threshold_type == "deterministic":
                ttype = "deterministic"
            elif m.require_threshold_type == "threshold":
                ttype = "threshold"
        elif pl.require_threshold_type is not None:
            verif.util.error("Internal error for output %s: Cannot understand required threshold type '%s'" % (pl.name(), pl.require_threshold_type))

        if ttype is not None:
            if ttype == "deterministic":
                smin = np.inf
                smax = -np.inf
                if verif.field.Obs() in data.get_fields():
                    obs = data.get_scores(verif.field.Obs(), 0)
                    smin = min(np.nanmin(obs), smin)
                    smax = max(np.nanmax(obs), smax)
                if verif.field.Fcst() in data.get_fields():
                    fcst = data.get_scores(verif.field.Fcst(), 0)
                    smin = min(np.nanmin(fcst), smin)
                    smax = max(np.nanmax(fcst), smax)
                num_default_thresholds = 20
                thresholds = np.linspace(smin, smax, num_default_thresholds)
                verif.util.warning("Missing '-r <thresholds>'. Automatically setting thresholds: %s" % (','.join(["%g" % q for q in thresholds])))
            elif ttype == "threshold":
                thresholds = data.thresholds
                verif.util.warning("Missing '-r <thresholds>'. Automatically setting thresholds: %s" % (','.join(["%g" % q for q in thresholds])))
            if len(thresholds) == 0:
                verif.util.error("No thresholds available")
    ttype = None
    if pl.require_threshold_type == "quantile":
        ttype = "quantile"
    elif m is not None:
        if m.require_threshold_type == "quantile":
            ttype = "quantile"

    if ttype == "quantile":
        if quantiles is None:
            quantiles = data.quantiles
            verif.util.warning("Missing '-q <quantiles>'. Automatically setting quantiles: %s" % (','.join(["%g" % q for q in quantiles])))

        # Check that we have the right number of quantiles
        if m is not None:
            if m.min_num_thresholds is not None:
                if len(quantiles) < m.min_num_thresholds:
                    verif.util.error(f"Need to specify at least {m.min_num_thresholds} quantiles (-q)")
            if m.max_num_thresholds is not None:
                if len(quantiles) > m.max_num_thresholds:
                    verif.util.error(f"Need to specify at most {m.max_num_thresholds} quantiles (-q)")

        # TODO: This is a bit of a hack, using thresholds to hold the
        # quantiles. But otherwise, the classes in output need to deal with
        # testing if the metric needs thresholds or quantiles
        thresholds = quantiles

    # Set plot parameters
    if simple is not None:
        pl.simple = simple
    if marker_sizes is not None:
        pl.ms = marker_sizes
    if line_widths is not None:
        pl.lw = line_widths
    if line_colors is not None:
        pl.line_colors = line_colors
    if line_styles is not None:
        pl.line_styles = line_styles
    if grid_style is not None:
        pl.grid_style = grid_style
    if grid_color is not None:
        pl.grid_color = grid_color
    if grid_width is not None:
        pl.grid_width = grid_width
    if markers is not None:
        pl.markers = markers
    if lab_font_size is not None:
        pl.labfs = lab_font_size
    if leg_font_size is not None:
        pl.legfs = leg_font_size
    if title_font_size is not None:
        pl.title_font_size = title_font_size
    if annotation_font_size is not None:
        pl.afs = annotation_font_size
    if leg_loc is not None:
        pl.leg_loc = leg_loc
    if tick_font_size is not None:
        pl.tick_font_size = tick_font_size
    if xrot is not None:
        pl.xrot = xrot
    if yrot is not None:
        pl.yrot = yrot
    if bottom_padding is not None:
        pl.bottom = bottom_padding
    if top_padding is not None:
        pl.top = top_padding
    if right_padding is not None:
        pl.right = right_padding
    if left_padding is not None:
        pl.left = left_padding
    if Pad is not None:
        pl.pad = None
    if bin_type is not None:
        pl.bin_type = bin_type
    if show_perfect is not None:
        pl.show_perfect = show_perfect
    if xlim is not None:
        pl.xlim = xlim
    if ylim is not None:
        pl.ylim = ylim
    if clim is not None:
        pl.clim = clim
    if xticks is not None:
        pl.xticks = xticks
    if xticklabels is not None:
        pl.xticklabels = xticklabels
    if yticks is not None:
        pl.yticks = yticks
    if yticklabels is not None:
        pl.yticklabels = yticklabels
    if xlog is not None:
        pl.xlog = xlog
    if ylog is not None:
        pl.ylog = ylog
    if annotate is not None:
        pl.annotate = annotate
    if annotation_format is not None:
        pl.annotation_format = annotation_format
    pl.grid = grid
    if cmap is not None:
        pl.cmap = cmap
    if map_type is not None:
        pl.map_type = map_type
    pl.filename = ofile
    if thresholds is not None:
        pl.thresholds = thresholds
    if quantiles is not None:
        pl.quantiles = quantiles
    pl.figsize = figsize
    pl.dpi = dpi
    if axis is not None:
        pl.axis = axis
    if aggregator_name is not None:
        pl.aggregator = verif.aggregator.get(aggregator_name)
    if aspect is not None:
        pl.aspect = aspect
    pl.show_margin = not no_margin
    if ylabel is not None:
        pl.ylabel = ylabel
    if xlabel is not None:
        pl.xlabel = xlabel
    if clabel is not None:
        pl.clabel = clabel
    if title is not None:
        pl.title = title
    if obs_leg is not None:
        pl.obs_leg = obs_leg
    if do_acc:
        if pl.supports_acc:
            pl.show_acc = do_acc
        else:
            verif.util.warning("%s does not support -acc" % metric)

    if plot_type == "text":
        pl.text(data)
    elif plot_type == "csv":
        pl.csv(data)
    elif plot_type == "map":
        pl.map(data)
    elif plot_type == "maprank":
        pl.show_rank = True
        pl.map(data)
    elif plot_type == "rank":
        pl.show_rank = True
        pl.plot_rank(data)
    elif plot_type == "impact":
        pl.plot_impact(data)
    elif plot_type == "mapimpact":
        pl.plot_mapimpact(data)
    else:
        pl.plot(data)


def get_aggregation_string():
    aggregators = verif.aggregator.get_all()
    value = "Aggregation type: "
    for aggregator in aggregators:
        if aggregator.name() != "quantile":
            value = "%s'%s', " % (value, aggregator.name())
    value = value + "or a number between 0 and 1. Some metrics computes a value for each value on the x-axis. Which function should be used to do the aggregation? Default is 'mean'. Only supported by some metrics. A number between 0 and 1 returns a specific quantile (e.g. 0.5 is the median)."
    return value


def show_description(data=None):
    desc = "Program to compute verification scores for weather forecasts. See https://github.com/WFRT/verif/wiki for how to format input files."
    s = "usage: verif files -m metric [options]\n"
    s += "\n"
    s += textwrap.fill(desc, get_text_width()) + "\n"
    s += "\n"
    s += verif.util.green("Arguments:") + "\n"
    s += format_argument("files", "One or more verification files in NetCDF or text format (see 'File Formats' below). The file format is autodetected.") + "\n"
    s += format_argument("-m metric", "Which verification metric to use? See 'Metrics' below.") + "\n"
    s += format_argument("--config file", "Read further arguments from this file. This flag can appear multiple times.") + "\n"
    s += format_argument("--list-times", "Prints what times are available in the files") + "\n"
    s += format_argument("--list-dates", "Like --list-times but in YYYYMMDD HH:MM:SS format") + "\n"
    s += format_argument("--list-locations", "Prints what locations are available in the files") + "\n"
    s += format_argument("--list-quantiles", "Prints what quantiles are available in the files") + "\n"
    s += format_argument("--list-thresholds", "Prints what thresholds are available in the files") + "\n"
    s += format_argument("--help", "Print this help message")

    s += format_argument("--version", "Prints what version of verif this is") + "\n"
    # Dimensions
    s += verif.util.green("  Dimensions and subset:") + "\n"
    s += "  (Note: vectors can be entered using commas, or MATLAB syntax i.e 3:5 is 3,4,5 and 3:2:7 is 3,5,7)\n"
    s += format_argument("-d dates", "A vector of dates in YYYYMMDD format, e.g.  20130101:20130201.") + "\n"
    s += format_argument("-elevrange range", "Limit the verification to locations within minelev,maxelev.") + "\n"
    s += format_argument("-l locations", "Limit the verification to these location IDs.") + "\n"
    s += format_argument("-lx locations", "Remove these locations from the verification. This happens after -l, -latrange, -lonrange, and -elevrange has been applied.") + "\n"
    s += format_argument("-latrange range", "Limit the verification to locations within minlat,maxlat.") + "\n"
    s += format_argument("-lonrange range", "Limit the verification to locations within minlon,maxlon.") + "\n"
    s += format_argument("-o leadtimes", "Limit the verification to these leadtimes (in hours).") + "\n"
    s += format_argument("-obsrange range", "Limit the verification to this range of observation values.") + "\n"
    s += format_argument("-r thresholds", "Compute scores using these thresholds (only used by some metrics).") + "\n"
    s += format_argument("-q quantiles", "Compute scores using these quantiles (only used by some metrics).") + "\n"
    s += format_argument("-t times", "A vector of unix timestamps. Only allow these times.") + "\n"
    s += format_argument("-tod timeofdays", "A vector of hours of day, e.g.  0:12. Only allow times with these hour of the day.") + "\n"
    s += format_argument("-x dim", "Plot this dimension on the x-axis: time, leadtime, year, month, week, day, timeofday, dayofyear, monthofyear, location, elev, lat, lon, threshold, leadtimeday, or no. Not supported by all metrics. If not specified, then a default is used based on the metric. 'location' refers to the location id. 'leadtimeday' aggregates leadtimes into whole forecast days. 'no' collapses all dimensions and computes one value.") + "\n"

    # Data manipulation
    s += verif.util.green("  Data manipulation:") + "\n"
    s += format_argument("-acc", "Accumulated values along the x-axis. Does not work for 'Special diagams.'") + "\n"
    s += format_argument("-agg type", get_aggregation_string()) + "\n"
    s += format_argument("-b type", "One of 'below' (< x), 'below=' (<= x), '=within' (<= x < ), 'within' (< x <), 'within=' (< x <=), '=within=' (<= x <=), 'above' (> x), or 'above=' (>= x). For threshold plots (ets, hit, within, etc) 'below/above' computes frequency below/above the threshold, and 'within' computes the frequency between consecutive thresholds.") + "\n"
    s += format_argument("-c file", "File containing climatology data. Subtract all forecasts and obs with climatology values.") + "\n"
    s += format_argument("-C file", "File containing climatology data. Divide all forecasts and obs by climatology values.") + "\n"
    s += format_argument("-fcst field", "What variable should be used as the forecast? 'obs', 'fcst' (default), threshold:<threshold>, quantile:<quantile>, 'pit', or the name of any other field in the input files.") + "\n"
    s += format_argument("-hist", "Plot values as histogram. Only works for any field that can be specified with -fcst.") + "\n"
    s += format_argument("-obs field", "What variable should be used as the observation? See -fcst.") + "\n"
    s += format_argument("-sort", "Plot values sorted. Only works for any field than can be specified with -fcst.") + "\n"
    s += format_argument("-T num", "Time-aggregate observations and forecasts across this many hours in time and leadtime.") + "\n"
    s += format_argument("-Tagg type", "Time-aggregate with this function (see -agg).") + "\n"
    s += format_argument("-Tx axis", "Time-aggregate across this axis (time or leadtime).") + "\n"

    # Plot options
    s += verif.util.green("  Plotting options:") + "\n"
    s += format_argument("-a", "Annotate graph by labeling each data point. Not supported by all metrics.") + "\n"
    s += format_argument("-af fields", "Show these fields in the annotation. Comma-separated list of one or more of: score,key,lat,lon,elev,location. 'score' is the metric's value, 'key' is the x-axis coordinate, and the others are valid for maps and where the x-axis is location related.") + "\n"
    s += format_argument("-afs size", "Set font size of annotations") + "\n"
    s += format_argument("-aspect ratio", "Force the aspect ratio of the plot. A value greater than 1 will stretch out the y-axis.") + "\n"
    s += format_argument("-bottom value", "Bottom boundary location for saved figure [range 0-1]") + "\n"
    s += format_argument("-clabel text", "Custom colorbar label") + "\n"
    s += format_argument("-clim limits", "Force colorbar limits to the two values lower,upper. Only used in combination with -type map.") + "\n"
    s += format_argument("-cmap colormap", "Use this colormap when possible (e.g. jet, inferno, RdBu). Only used in combination with -type map.") + "\n"
    s += format_argument("-dpi value", "Resolution of image in dots per inch (default 100)") + "\n"
    s += format_argument("-gc color", "Color for grid lines. E.g red,[0.3,0,0],0.3.") + "\n"
    s += format_argument("-gs style", "Line styles for gri, such as -, --, or :.") + "\n"
    s += format_argument("-gw width", "Line width for grid") + "\n"
    s += format_argument("-f file", "Save image to this filename") + "\n"
    s += format_argument("-fs size", "Set figure size width,height (in inches). Default 8x6.") + "\n"
    s += format_argument("-labfs size", "Font size for axis labels") + "\n"
    s += format_argument("-lc colors", "Comma-separated list of line colors, such as red,[0.3,0,0],0.3. Colors are repeated if there are more lines than colors.") + "\n"
    s += format_argument("-left value", "Left boundary location for saved figure [range 0-1]") + "\n"
    s += format_argument("-leg titles", "Comma-separated list of legend titles. Use '_' to represent space.") + "\n"
    s += format_argument("-legfs size", "Font size for legend. Set to 0 to hide legend.") + "\n"
    s += format_argument("-legloc loc", "Where should the legend be placed?  Locations such as 'best', 'upper_left', 'lower_right', 'center'. Use underscore when using two words.") + "\n"
    s += format_argument("-ls styles", "Comma-separated list of line styles, such as -,--. Styles are repeated if there are more lines than styles.") + "\n"
    s += format_argument("-lw width", "Comma-separated list of line widths") + "\n"
    s += format_argument("-maptype type", "One of 'simple', 'sat', or 'topo'.  'simple' shows a basic ocean/lakes/land map, 'sat' shows a satellite image, and 'topo' a topographical map. Only relevant when '-type map' has been selected.") + "\n"
    s += format_argument("-ma markers", "Comma-separated list of markers (e.g.  o,*,x)") + "\n"
    s += format_argument("-ms size", "Comma-separated list of marker sizes") + "\n"
    s += format_argument("-nogrid", "Turn the grid on the plot off") + "\n"
    s += format_argument("-nomargin", "Remove margins (whitespace) in the plot") + "\n"
    s += format_argument("-obsleg", "Name to put in legend for observations (if applicable)") + "\n"
    s += format_argument("-right value", "Right boundary location for saved figure [range 0-1]. Must be greater than -left.") + "\n"
    s += format_argument("-simple", "Make a simpler plot, without extra lines, subplots, etc.") + "\n"
    s += format_argument("-sp", "Show a line indicating the perfect score") + "\n"
    s += format_argument("-tickfs size", "Font size for axis ticks") + "\n"
    s += format_argument("-title text", "Custom title to chart top") + "\n"
    s += format_argument("-titlefs size", "Font size for title.") + "\n"
    s += format_argument("-top value", "Top boundary location for saved figure [range 0-1].  Must be greater than -bottom.") + "\n"
    s += format_argument("-type type", "One of 'plot' (default), 'text', 'csv', 'map', 'rank', 'maprank', 'impact', or 'mapimpact'.") + "\n"
    s += format_argument("-xlabel text", "Custom x-axis label") + "\n"
    s += format_argument("-xlim limits", "Force x-axis limits to the two values lower,upper") + "\n"
    s += format_argument("-xlog", "Use a logarithmic x-axis") + "\n"
    s += format_argument("-xrot value", "Rotation angle for x-axis labels") + "\n"
    s += format_argument("-xticks ticks", "A vector of values to put ticks on the x-axis") + "\n"
    s += format_argument("-xticklabels labs", "A comma-separated list of labels for the x-axis ticks") + "\n"
    s += format_argument("-ylabel text", "Custom y-axis label") + "\n"
    s += format_argument("-ylim limits", "Force y-axis limits to the two values lower,upper") + "\n"
    s += format_argument("-ylog", "Use a logarithmic y-axis") + "\n"
    s += format_argument("-yrot value", "Rotation angle for y-axis labels") + "\n"
    s += format_argument("-yticks ticks", "A vector of values to put ticks on the y-axis") + "\n"
    s += format_argument("-yticklabels labs", "A comma-separated list of labels for the y-axis ticks") + "\n"
    s += "\n"
    s += verif.util.green("Metrics (-m):") + "\n"
    s += "  (For a full description of a metric, run verif -m <metric>)\n"
    metric_types = [verif.metric_type.Deterministic(),
          verif.metric_type.Threshold(),
          verif.metric_type.Probabilistic(),
          verif.metric_type.Diagram()]
    for metric_type in metric_types:
        metrics = verif.metric.get_all_by_type(metric_type)
        outputs = verif.output.get_all_by_type(metric_type)
        metric_outputs = metrics + outputs
        metric_outputs.sort(key=lambda x: x[0].lower(), reverse=False)
        if len(metric_outputs) > 0:
            s += verif.util.green("  %s:" % metric_type.description) + "\n"
            for m in metric_outputs:
                name = m[0].lower()
                if m[1].is_valid():
                    desc = m[1].description
                    s += format_argument(name, desc) + "\n"

    return s


def format_argument(arg, description, arg_width=22, total_width=None, indent=2):
    """
    Prints formated description to screen, but adds a column for a short descriptor, like this:
                arg            description more description
                               here more more more more more
    | indent | | arg_width   | | total_width                                 |
    """
    if total_width is None:
        total_width = get_text_width()
    fmt = "%-" + str(indent) + "s%-" + str(arg_width - indent) + "s"
    curr = fmt % ("", arg)
    if len(arg) > arg_width - indent - 2:
        output = curr + '\n'
        curr = ""
        for i in range(0, arg_width):
            curr = curr + " "
    else:
        output = ""
    lines = description.split('\n')
    for line_num in range(0, len(lines)):
        line = lines[line_num]
        words = line.split()
        for i in range(0, len(words)):
            word = words[i]
            if len(curr) + len(word) >= total_width:
                output = output + curr + "\n"
                curr = ""
                for i in range(0, arg_width):
                    curr = curr + " "
            elif i != 0:
                curr = curr + " "
            curr = curr + word
        output = output + curr
        if line_num < len(lines)-1:
            output = output + "\n"
        curr = " " * arg_width
    return output


def get_text_width():
    """ How wide should the text be output? """
    # return max(50, min(100, get_screen_width()))
    return 80


if __name__ == '__main__':
    run(sys.argv)



from __future__ import print_function
import six

import datetime
import matplotlib.dates
import numpy as np
import unittest
import verif.util


class TestApplyThreshold(unittest.TestCase):
    def test_1(self):
        ar = np.array([1, 3, 2])
        np.testing.assert_array_equal(
            np.array([0, 1, 0]), verif.util.apply_threshold(ar, "above", 2)
        )
        np.testing.assert_array_equal(
            np.array([0, 1, 1]), verif.util.apply_threshold(ar, "above=", 2)
        )
        np.testing.assert_array_equal(
            np.array([1, 0, 0]), verif.util.apply_threshold(ar, "below", 2)
        )
        np.testing.assert_array_equal(
            np.array([1, 0, 1]), verif.util.apply_threshold(ar, "below=", 2)
        )

    def test_same_as_intervals(self):
        ar = np.array([1, 3, 2])
        interval = verif.interval.Interval(2, np.inf, False, False)
        np.testing.assert_array_equal(
            interval.within(ar), verif.util.apply_threshold(ar, "above", 2)
        )
        interval = verif.interval.Interval(2, np.inf, True, False)
        np.testing.assert_array_equal(
            interval.within(ar), verif.util.apply_threshold(ar, "above=", 2)
        )
        interval = verif.interval.Interval(-np.inf, 2, False, False)
        np.testing.assert_array_equal(
            interval.within(ar), verif.util.apply_threshold(ar, "below", 2)
        )
        interval = verif.interval.Interval(-np.inf, 2, False, True)
        np.testing.assert_array_equal(
            interval.within(ar), verif.util.apply_threshold(ar, "below=", 2)
        )


class TestParseNumbers(unittest.TestCase):
    def test_simple(self):
        self.assertEqual([2], verif.util.parse_numbers("2"))
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("test")

    def test_empty(self):
        invalids = ["", "1,", "1,,2"]
        for invalid in invalids:
            with self.assertRaises(SystemExit):
                verif.util.parse_numbers(invalid)

    def test_vector(self):
        self.assertEqual([2, 3, 4, 5], verif.util.parse_numbers("2:5"))
        self.assertEqual([], verif.util.parse_numbers("2:1"))
        with self.assertRaises(SystemExit):
            self.assertEqual([2], verif.util.parse_numbers("2:test"))

    def test_vectorInc(self):
        self.assertEqual([2, 5, 8], verif.util.parse_numbers("2:3:8"))
        self.assertEqual([2, 5], verif.util.parse_numbers("2:3:7"))
        self.assertEqual([], verif.util.parse_numbers("2:-1:7"))
        self.assertEqual([2, 1, 0], verif.util.parse_numbers("2:-1:0"))
        self.assertEqual([8, 5, 2], verif.util.parse_numbers("8:-3:0"))
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2:3:test")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("test:3:5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2:test:5")

    def test_comma(self):
        self.assertEqual([2, 5], verif.util.parse_numbers("2,5"))
        self.assertEqual([3, 3], verif.util.parse_numbers("3,3"))
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("test")

    def test_decimal(self):
        self.assertEqual(
            [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7], verif.util.parse_numbers("0.1:0.1:0.7")
        )
        self.assertEqual(
            [1e-4, 2e-4, 3e-4, 4e-4, 5e-4, 6e-4, 7e-4],
            verif.util.parse_numbers("0.0001:0.0001:0.0007"),
        )
        self.assertEqual([1e-6, 3e-6], verif.util.parse_numbers("0.000001,0.000003"))

    def test_0_step(self):
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("0:0:5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("0:0:0")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("0:0:-5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("20150101:0:20150101", is_date=True)
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("20150101:0:20150105", is_date=True)
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("20150101:0:20140101", is_date=True)

    def test_mix(self):
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2,5:8,3,test")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2,5:8,test,5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2,5:test,3,5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2,5:test,3,5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("2,test:8,3,5")
        with self.assertRaises(SystemExit):
            verif.util.parse_numbers("test,5:8,3,5")

    def test_date(self):
        self.assertEqual(
            [20141230, 20141231, 20150101, 20150102, 20150103],
            verif.util.parse_numbers("20141230:20150103", True),
        )
        self.assertEqual(
            [20141230, 20150101, 20150103],
            verif.util.parse_numbers("20141230:2:20150104", True),
        )

        self.assertEqual(
            [20141230, 20141231, 20150101, 20150102, 20150103],
            verif.util.parse_dates("20141230:20150103"),
        )
        self.assertEqual(
            [20141230, 20150101, 20150103],
            verif.util.parse_dates("20141230:2:20150104"),
        )

    def test_unixtime_to_datenum(self):
        unixtimes = np.array([1331856000, -2180131200])
        datenums = [verif.util.unixtime_to_datenum(unixtime) for unixtime in unixtimes]
        self.assertEqual(
            matplotlib.dates.date2num(datetime.datetime(2012, 3, 16, 0, 0)), datenums[0]
        )
        self.assertEqual(
            matplotlib.dates.date2num(datetime.datetime(1900, 12, 1, 0, 0)), datenums[1]
        )

    def test_date_to_unixtime(self):
        import time

        s = time.time()
        self.assertEqual(1475280000, verif.util.date_to_unixtime_slow(20161001))
        e = time.time()
        print(e - s)
        s = time.time()
        self.assertEqual(1475280000, verif.util.date_to_unixtime(20161001))
        e = time.time()
        print(e - s)

    def test_unixtime_to_date(self):
        self.assertEqual(20161001, verif.util.unixtime_to_date(1475280000))


class TestGetDate(unittest.TestCase):
    def test_simple(self):
        self.assertEqual(20150207, verif.util.get_date(20150206, 1))
        self.assertEqual(20150205, verif.util.get_date(20150206, -1))
        self.assertEqual(20150215, verif.util.get_date(20150210, 5))

    def test_endofyear(self):
        self.assertEqual(20150101, verif.util.get_date(20141231, 1))
        self.assertEqual(20141226, verif.util.get_date(20150105, -10))
        self.assertEqual(20150105, verif.util.get_date(20141226, 10))
        self.assertEqual(20141231, verif.util.get_date(20150101, -1))


class TestGetIntervals(unittest.TestCase):
    def test_above(self):
        thresholds = [0, 1, 5]
        intervals = verif.util.get_intervals("above", thresholds)
        self.assertEqual(verif.interval.Interval(0, np.inf, False, False), intervals[0])

    def test_within(self):
        thresholds = [0, 1, 5]
        intervals = verif.util.get_intervals("within", thresholds)
        self.assertEqual(2, len(intervals))
        self.assertEqual(verif.interval.Interval(0, 1, False, False), intervals[0])
        self.assertEqual(verif.interval.Interval(1, 5, False, False), intervals[1])

    def test_within_single(self):
        thresholds = [0]
        intervals = verif.util.get_intervals("within", thresholds)
        self.assertEqual(0, len(intervals))


class TestAlmostEqual(unittest.TestCase):
    def test_1(self):
        for tol in [10 ** (-i) for i in range(1, 10)]:
            self.assertTrue(verif.util.almost_equal(0, 0, tol))
            self.assertTrue(verif.util.almost_equal(3, 3, tol))
            self.assertTrue(verif.util.almost_equal(-2.3, -2.3, tol))
            self.assertFalse(verif.util.almost_equal(-2.3, 2.3, tol))
            self.assertFalse(verif.util.almost_equal(-2.3, 0, tol))
            self.assertFalse(verif.util.almost_equal(0, 15.2, tol))

        self.assertTrue(verif.util.almost_equal(3.12345678, 3.12345679, 1e-1))
        self.assertTrue(verif.util.almost_equal(3.12345678, 3.12345679, 1e-5))
        self.assertTrue(verif.util.almost_equal(3.12345678, 3.12345679, 1e-8))
        self.assertFalse(verif.util.almost_equal(3.12345678, 3.12345679, 1e-9))
        self.assertTrue(verif.util.almost_equal(3.123, 3.1241, 1e-1))
        self.assertTrue(verif.util.almost_equal(3.123, 3.1241, 1e-2))
        self.assertFalse(verif.util.almost_equal(3.123, 3.1241, 1e-3))


class TestConvert(unittest.TestCase):
    def test_convert_back_and_forth(self):
        dates = [20150101, 20141231]
        new_dates = [
            verif.util.datenum_to_date(verif.util.date_to_datenum(date))
            for date in dates
        ]
        six.assertCountEqual(self, new_dates, dates)


class TestIsValidNc(unittest.TestCase):
    def test_1(self):
        # self.assertTrue(verif.util.is_valid_nc("verif/tests/files/netcdf_valid1.nc"))
        self.assertFalse(verif.util.is_valid_nc("verif/tests/files/file1.txt"))


class TestProj4(unittest.TestCase):
    def test_1(self):
        string = (
            "+proj=lcc +lat_0=63 +lon_0=15 +lat_1=63 +lat_2=63 +no_defs +R=6.371e+07"
        )
        r = verif.util.proj4_string_to_dict(string)
        self.assertEqual(r.get("+proj"), "lcc")
        self.assertEqual(r.get("+lat_0"), 63)
        self.assertEqual(r.get("+no_defs"), True)
        self.assertEqual(r.get("+lon_1"), None)


class TestDistance(unittest.TestCase):
    def test_1(self):
        # Verified against http://www.nhc.noaa.gov/gccalc.shtml
        # They only give answer to nearest km, so allow some deviation
        self.assertEqual(0, verif.util.distance(60, 10, 60, 10))
        self.assertLess(abs(1360000 - verif.util.distance(50.5, 3.4, 61.9, 11.5)), 2000)
        self.assertLess(
            abs(15712000 - verif.util.distance(-47.2, -24.4, 82.1, 101.5)), 2000
        )


class Test(unittest.TestCase):
    def test_numvalid(self):
        data = np.array([[0, 1, 2], [10, 11.0, 12]])
        self.assertEqual(6, verif.util.numvalid(data))
        np.testing.assert_array_almost_equal([2, 2, 2], verif.util.numvalid(data, 0))
        np.testing.assert_array_almost_equal([3, 3], verif.util.numvalid(data, 1))

    def test_nprange(self):
        data = np.array([[0, 1, 2], [10, 11.0, 12]])
        self.assertEqual(12, verif.util.nprange(data))
        np.testing.assert_array_almost_equal([10, 10, 10], verif.util.nprange(data, 0))
        np.testing.assert_array_almost_equal([2, 2], verif.util.nprange(data, 1))


if __name__ == "__main__":
    unittest.main()



import os
import sys
import numpy as np
import matplotlib.pylab as mpl
import argparse
import verif.util
import netCDF4
import shutil
import scipy.interpolate


def main():
    """
    It takes an existing Verif file, extracts 6h forecasts, and fills in the intermediate hours with
    either linearly or splin interpolated values.

    Ir interpolates fcst, ensemble, ensemble_mean, ensmble_variable fields and creates
    ensemble_crps.
    """
    parser = argparse.ArgumentParser(description='This program downsamples a verif file and interpolates the data back using an interpolation method')
    parser.add_argument('ifile', help='')
    parser.add_argument('ofile', help='')
    parser.add_argument('-i', choices=['linear', 'spline'], dest='interpolator', required=True)
    parser.add_argument('-f', type=int, dest="frequency", required=True)

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    args = parser.parse_args()
    frequency = args.frequency

    if frequency < 2:
        raise ValueError("frequency must be > 1")

    shutil.copy(args.ifile, args.ofile)

    with netCDF4.Dataset(args.ofile, 'a') as file:
        leadtimes = file.variables["leadtime"][:]
        num_leadtimes = len(leadtimes)

        has_ens = "ensemble" in file.variables
        if has_ens:
            ensemble = file.variables["ensemble"][:]
            print(np.sum(np.isnan(ensemble)))

        if frequency >= num_leadtimes:
            raise ValueError(f"frequency must be < {num_leadtimes}")

        fcst = file.variables["fcst"][:]
        print("Interpolating... ")

        if args.interpolator == "linear":
            # Linear interpolation works, even if there are NaNs in the dataset, the result will
            # just be NaNs, as expected.
            ends = range(frequency, num_leadtimes+1, frequency)
            for end in ends:
                start = end - frequency

                fcst_start = fcst[:, start, :]
                fcst_end = fcst[:, end, :]
                fcst[:, (start+1):end, :] = np.nan  # Erase everything in the middle
                for i in range(1, frequency):
                    w1 = i / frequency
                    w0 = 1 - w1
                    fcst[:, start + i, :] = w0 * fcst_start + w1 * fcst_end

                if has_ens:
                    ensemble[:, (start+1):end, :, :] = np.nan
                    fcst_start = ensemble[:, start, :, :]
                    fcst_end = ensemble[:, end, :]
                    for i in range(1, frequency):
                        w1 = i / frequency
                        w0 = 1 - w1
                        ensemble[:, start + i, :, :] = w0 * fcst_start + w1 * fcst_end

            # Trim the last leadtimes that cannot be interpolated
            fcst[:, ends[-1]+1:, :] = np.nan
            if has_ens:
                ensemble[:, ends[-1]+1:, :, :] = np.nan

        elif args.interpolator == "spline":
            y = fcst[:, 0::frequency, :]
            x = leadtimes[0::frequency]

            interpolator = scipy.interpolate.CubicSpline(x, y, axis=1)
            xx = set([i for i in leadtimes]) - set([i for i in x])
            xx = np.array(list(xx))
            temp = interpolator(xx)
            for i, index in enumerate(xx):
                fcst[:, int(index), :] = temp[:, i, :]

            if has_ens:
                y = ensemble[:, 0::frequency, :, :]
                x = leadtimes[0::frequency]

                # Spline can't deal with NaNs in y, so set NaNs to a 0 and then put the Nans back in
                # later. This means that if a single timestep is missing for a member, the whole
                # member is NaN for that time/location.
                is_nan = np.sum(np.isnan(y), axis=1) > 0
                y[np.isnan(y)] = 0

                interpolator = scipy.interpolate.CubicSpline(x, y, axis=1)
                xx = set([i for i in leadtimes]) - set([i for i in x])
                xx = np.array(list(xx))
                interp = interpolator(xx)
                for i, index in enumerate(xx):
                    temp = interp[:, i, :, :]
                    temp[is_nan] = np.nan
                    ensemble[:, int(index), :, :] = temp

        file.variables["fcst"][:] = fcst
        if has_ens:
            file.variables["ensemble"][:] = ensemble

        # These variables are no longer valid, since we cannot derive them for interpolated
        # timeseries
        invalid_variables = ["x", "cdf", "pdf", "pit", "ensemble_mean", "ensemble_variance"]
        for variable in invalid_variables:
            if variable in file.variables:
                file.variables[variable] = np.nan

        # Compute CRPS
        if has_ens:
            print("Computing CRPS...")
            if not "ensemble_crps" in file.variables:
                file.createVariable("ensemble_crps", "f4", ("time", "leadtime", "location"))

            num_members = ensemble.shape[-1]
            crps = compute_crps(ensemble, file.variables["obs"][:], num_members)

            num_valid_members = np.sum(~np.isnan(ensemble), axis=3)
            crps[num_valid_members != num_members] = np.nan

            file.variables["ensemble_crps"][:] = crps

            if "ensemble_mean" in file.variables:
                ens_mean = np.nanmean(ensemble, axis=3)
                file.variables["ensemble_mean"][:] = ens_mean
            if "ensemble_variance" in file.variables:
                file.variables["ensemble_mean"][:] = np.nanvar(ensemble, axis=3)


def compute_crps(preds, targets, num_members, fair=True) -> np.ndarray:
    """Continuous Ranked Probability Score (CRPS).

    Taken from bris-inference

    Args:
        preds: numpy.ndarray
            Predictions, shape (time, leadtime, location, ens_size)
        targets: numpy.ndarray
            Targets, shape (time, leadtime, location)
        fair: bool
            Defaults to true

    Returns:
        crps: numpy.ndarray
            Shape (time, leadtime, location)
    """

    coef = (
        -1.0 / (num_members * (num_members - 1))
        if fair
        else -1.0 / (num_members**2)
    )

    mae = np.mean(np.abs(targets[..., None] - preds), axis=-1)

    # var = np.abs(preds[..., None] - preds[..., None, :])
    var = np.zeros(preds.shape[:-1])
    for i in range(num_members):  # loop version to reduce memory usage
        var += np.sum(np.abs(preds[..., i, None] - preds[..., i + 1 :]), axis=-1)
    var *= coef
    return mae + var


if __name__ == "__main__":
    main()



# Taken from https://gist.github.com/swenson/8142788
import os
import os.path
import unittest
import pep8
import warnings

# ignore stuff in virtualenvs or version control directories
ignore_patterns = (
    ".svn",
    "bin",
    "lib" + os.sep + "python",
    "verif/devel",
    "tests/devel",
    "devel",
)


def ignore(dir):
    """Should the directory be ignored?"""
    for pattern in ignore_patterns:
        if pattern in dir:
            return True
    return False


class TestPep8(unittest.TestCase):
    def test_pep8(self):
        style = pep8.StyleGuide(quiet=False)
        style.options.ignore += ("E501",)
        style.options.ignore += ("E114",)
        style.options.ignore += ("E121",)
        style.options.ignore += ("E122",)
        style.options.ignore += ("E126",)
        style.options.ignore += ("E127",)
        style.options.ignore += ("E128",)
        # style.options.max_line_length = 100  # because it isn't 1928 anymore

        errors = 0
        for dir in ["verif/", "verif/tests/", "scripts/"]:
            for root, _, files in os.walk(dir):
                if ignore(root):
                    continue

                python_files = [
                    os.path.join(root, f) for f in files if f.endswith(".py")
                ]
                report = style.check_files(python_files)
                errors += report.total_errors

        if errors > 0:
            warnings.warn(
                "Warning: There are %d PEP8 style errors in the source files" % errors
            )


if __name__ == "__main__":
    unittest.main()



import unittest
import verif.field


class TestField(unittest.TestCase):
    def test_equal(self):
        self.assertEqual(verif.field.Obs(), verif.field.Obs())
        self.assertTrue(verif.field.Obs() == verif.field.Obs())
        self.assertFalse(verif.field.Obs() != verif.field.Obs())

        self.assertEqual(verif.field.Threshold(0), verif.field.Threshold(0))
        self.assertTrue(verif.field.Threshold(0) == verif.field.Threshold(0))
        self.assertFalse(verif.field.Threshold(0) != verif.field.Threshold(0))

    def test_unequal(self):
        self.assertFalse(verif.field.Obs() == verif.field.Fcst())
        self.assertFalse(verif.field.Obs() == verif.field.Threshold(0))
        self.assertTrue(verif.field.Obs() != verif.field.Fcst())
        self.assertTrue(verif.field.Obs() != verif.field.Threshold(0))

        self.assertFalse(verif.field.Threshold(0) == verif.field.Threshold(1))
        self.assertTrue(verif.field.Threshold(0) != verif.field.Threshold(1))

    def test_get(self):
        f = verif.field.get("threshold:0.1")
        self.assertEqual(f, verif.field.Threshold(0.1))

        f = verif.field.get("quantile:0.1")
        self.assertEqual(f, verif.field.Quantile(0.1))

        f = verif.field.get("ensemble:1")
        self.assertEqual(f, verif.field.EnsembleMember(1))


if __name__ == "__main__":
    unittest.main()



import os
import re
import sys
import matplotlib.pylab as mpl
import verif.driver
import verif.metric
import verif.output

"""
This script creates a bash completion script by parsing command-line options from verif's
description as well as reading all classed form verif.metric and verif.output
"""

description = verif.driver.show_description()
lines = description.split('\n')
reg = re.compile("^  -")
lines = [line for line in lines if reg.match(line)]
for i in range(0, len(lines)):
    line = lines[i]
    line = line.split(' ')
    line = [q for q in line if q != '']
    lines[i] = line[0]

aggregators = [agg.name() for agg in verif.aggregator.get_all()]
aggregators.sort()

axes = [axis[0].lower() for axis in verif.axis.get_all()]
axes.sort()

metrics = verif.metric.get_all()
outputs = verif.output.get_all()
metricOutputs = metrics + outputs
metricOutputs.sort(key=lambda x: x[0].lower(), reverse=False)

types = ["csv", "plot", "text", "map", "rank", "maprank", "impact", "mapimpact"]
types.sort()

print("# START verif completion")
print("_verif()")
print("{")
print("local cur prev opts")

print('COMPREPLY=()')
print('cur="${COMP_WORDS[COMP_CWORD]}"')
print('prev="${COMP_WORDS[COMP_CWORD-1]}"')

# Files
print('if [ "$cur" = "" ] || [[ "$cur" =~ -* ]]; then')
print("   COMPREPLY=( $( compgen -f -W '",  end = '')
string = ' '.join(lines)
print(string, end = '')
print("' -- $cur ) )")
print('fi')

# Metrics
print('if [ "$prev" = "-m" ]; then')
print("   COMPREPLY=( $( compgen -W '", end = '')
for m in metricOutputs:
    name = m[0].lower()
    if(m[1].is_valid()):
        desc = m[1].get_class_name()
        print(name + " ", end = '')
print("' -- $cur ) )")

# Cmap
print('elif [ "$prev" = "-cmap" ]; then')
print("   COMPREPLY=( $( compgen -W '", end = '')
cmaps = list(mpl.colormaps.keys())
cmaps.sort()
print(' '.join(cmaps), end = '')
print("' -- $cur ) )")

# Agg
print('elif [ "$prev" = "-agg" ] || [ "$prev" = "-Tagg" ]; then')
print("   COMPREPLY=( $( compgen -W '", end = '')
print(' '.join(aggregators), end = '')
print("' -- $cur ) )")

# Type
print('elif [ "$prev" = "-type" ]; then')
print("   COMPREPLY=( $( compgen -W '", end = '')
print(' '.join(types), end = '')
print("' -- $cur ) )")

# Axis
print('elif [ "$prev" = "-x" ]; then')
print("   COMPREPLY=( $( compgen -W '", end = '')
print(' '.join(axes), end = '')
print("' -- $cur ) )")
print('elif [ "$prev" = "-Tx" ]; then')
print("   COMPREPLY=( $( compgen -W '", end = '')
print(" leadtime time ' -- $cur ) )")
print('fi')
print('return 0')
print('}')
print('complete -ofilenames -F _verif verif')
print('# END verif completion')



import unittest
import verif.axis


class TestAxis(unittest.TestCase):
    def test_equal(self):
        self.assertEqual(verif.axis.No(), verif.axis.No())
        self.assertTrue(verif.axis.No() == verif.axis.No())
        self.assertFalse(verif.axis.No() != verif.axis.No())

    def test_unequal(self):
        self.assertFalse(verif.axis.No() == verif.axis.Location())
        self.assertFalse(verif.axis.No() == verif.axis.Time())
        self.assertTrue(verif.axis.No() != verif.axis.Location())
        self.assertTrue(verif.axis.No() != verif.axis.Time())


if __name__ == "__main__":
    unittest.main()



import numpy as np
import verif.util


class Location(object):
    """ Class for storing a geographical point, as well as its metadata

    Attributes:
       id (int): station identifier
       lat (float): decimal degrees latitude
       lon (float): decimal degrees longitude
       elev (float): elevation in meters
    """

    radius_earth = 6.371e6

    def __init__(self, id, lat, lon, elev):
        self.id = id
        self.lat = lat
        self.lon = lon
        self.elev = elev

    def get_distance(self, other):
        """ Calculate the distance between this location and another location

        Arguments:
           other (Location): The other location

        Returns:
           float: Distance in meters
        """
        lat1 = self.lat
        lon1 = self.lon
        lat2 = other.lat
        lon2 = other.lon
        if lat1 == lat2 and lon1 == lon2:
            return 0

        lat1r = verif.util.deg2rad(lat1)
        lat2r = verif.util.deg2rad(lat2)
        lon1r = verif.util.deg2rad(lon1)
        lon2r = verif.util.deg2rad(lon2)
        ratio = np.cos(lat1r) * np.cos(lon1r) * np.cos(lat2r) * np.cos(lon2r) +\
                np.cos(lat1r) * np.sin(lon1r) * np.cos(lat2r) * np.sin(lon2r) +\
                np.sin(lat1r) * np.sin(lat2r)
        if ratio > 1:
            ratio = 1
        dist = np.arccos(ratio) * self.radius_earth

        return dist

    def __eq__(self, other):
        if np.isnan(self.id) and np.isnan(other.id):
            same_id = True
        else:
            same_id = self.id == other.id
        same_lat = verif.util.almost_equal(self.lat, other.lat, 1e-5)
        same_lon = verif.util.almost_equal(self.lon, other.lon, 1e-5)
        same_elev = verif.util.almost_equal(self.elev, other.elev, 1e-5)
        return same_id and same_lat and same_lon and same_elev

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash((self.lat, self.lon, self.elev))

    def __str__(self):
        return "(id,lat,lon,elev) = (%g,%g,%g,%g)" % (self.id, self.lat, self.lon, self.elev)



import re
import verif.metric
import verif.output

metrics = [metric for metric in verif.metric.get_all() if metric[1].is_valid()]
outputs = [output for output in verif.output.get_all() if output[1].is_valid()]

print("Metrics: %d" % len(metrics))
print("Outputs: %d" % len(outputs))
print("Total: %d" % (len(metrics) + len(outputs)))

description = verif.driver.show_description()
lines0 = description.split('\n')
reg = re.compile("^  -")
lines0 = [line for line in lines0 if reg.match(line)]
lines = list()
for i, line in enumerate(lines0):
    line = line.split(' ')
    line = [q for q in line if q != '']
    line = line[0]
    if line[0:2] == '--':
        continue
    lines += [line]
print("(real) options: %d" % len(lines))



import argparse
import copy
import netCDF4
import numpy as np
import scipy.interpolate
import sys


import verif.input


def main():
    parser = argparse.ArgumentParser(prog="ens2prob", description="Compute probabilistic information from ensembles")
    parser.add_argument('ifile', help="Verif text or NetCDF file (input)")
    parser.add_argument('ofile', help="Verif NetCDF file (output)")
    parser.add_argument('-r', default=[], type=verif.util.parse_numbers, help="Which thresholds (e.g.  temperatures) to compute CDF values for?", dest="thresholds")
    parser.add_argument('-q', default=[], type=verif.util.parse_numbers, help="Which quantiles (between 0 and 1) to compute?", dest="quantiles")
    parser.add_argument('-p', help="Compute PIT values?", dest="pit", action="store_true")

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(0)

    args = parser.parse_args()

    ifile = verif.input.get_input(args.ifile)
    locations = ifile.locations
    locationids = [loc.id for loc in locations]
    leadtimes = ifile.leadtimes
    times = ifile.times
    lats = [loc.lat for loc in locations]
    lons = [loc.lon for loc in locations]
    elevs = [loc.elev for loc in locations]

    """ Initialize """
    fcst = copy.deepcopy(ifile.fcst)
    obs = copy.deepcopy(ifile.obs)
    ens = copy.deepcopy(ifile.ensemble)
    ens = np.sort(ens, axis=3)
    if len(args.thresholds) > 0:
        cdf = np.zeros([obs.shape[0], obs.shape[1], obs.shape[2], len(args.thresholds)])
    if len(args.quantiles) > 0:
        x = np.zeros([obs.shape[0], obs.shape[1], obs.shape[2], len(args.quantiles)])
    if args.pit:
        pit = np.nan * np.zeros([obs.shape[0], obs.shape[1], obs.shape[2]])

    M = ens.shape[3]

    """
    Two approaches to assigning probabilities to ensemble members:
    1) Assign a cumulative probability of 0 to the lowest member and 1 to the highest
    2) Assign a cumulative probability of 1 / (M+1) to the lowest and M / (M+1) to the highest
    In either case, if a value is outside the ensemble range, set the value to the CDF assigned to
    the half of its nearest member.

    Then there are two approach for interpolating probabilities:
    a) round down to the nearest member
    b) linearly interpolate

    """
    lower_cdf = 0  # 1.0 / (M + 1)
    upper_cdf = 1 - lower_cdf

    """
    Compute cumulative probabilities at different thresholds

    Currently, only approach a) is supported.
    """
    for i, threshold in enumerate(args.thresholds):
        # Equality operator doesn't handle missing values
        I = np.isnan(ens) == 0
        cond = np.nan * np.zeros(ens.shape)
        cond[I] = ens[I] < threshold
        cdf[:, :, :, i] = np.nanmean(cond, axis=3) * (upper_cdf - lower_cdf) + lower_cdf / 2

    """
    Compute values for different quantiles

    Use kind='linear' for approach b)
    """
    for i, quantile in enumerate(args.quantiles):
        f = scipy.interpolate.interp1d(np.linspace(lower_cdf, upper_cdf, M),
                ens, bounds_error=False,
                axis=3, kind='zero')
        if quantile == 1:
            x[:, :, :, i] = ens[:, :, :, -1]
        else:
            x[:, :, :, i] = f(quantile)

    """
    Compute PIT values, i.e. the CDF at the observed value

    Currently, only approach a) is supported.
    """
    if args.pit:
        if obs is None:
            print("Error: File is missing obs, and can therefore not compute PIT")
            sys.exit(1)
        newobs = np.tile(np.expand_dims(obs, 3), [1, 1, 1, M])
        pit = np.mean(ens < newobs, axis=3)
        """
        # approach b)
        for i in range(0, obs.shape[0]):
            for j in range(0, obs.shape[1]):
                for t in range(0, obs.shape[2]):
                    if np.isnan(obs[i, j , t]) == 0:
                        pit[i, j, t] = np.interp(obs[i, j, t], ens[i, j, t, :], np.linspace(lower_cdf, upper_cdf, M), left=lower_cdf/2, right=1-(1-upper_cdf)/2)
        """

    file = netCDF4.Dataset(args.ofile, 'w', format="NETCDF4")
    file.createDimension("leadtime", len(ifile.leadtimes))
    file.createDimension("time", None)
    file.createDimension("location", len(ifile.locations))
    if len(args.thresholds) > 0:
        file.createDimension("threshold", len(args.thresholds))
    if len(args.quantiles) > 0:
        file.createDimension("quantile", len(args.quantiles))
    vTime = file.createVariable("time", "i4", ("time",))
    vOffset = file.createVariable("leadtime", "f4", ("leadtime",))
    vLocation = file.createVariable("location", "f8", ("location",))
    vLat = file.createVariable("lat", "f4", ("location",))
    vLon = file.createVariable("lon", "f4", ("location",))
    vElev = file.createVariable("altitude", "f4", ("location",))
    if fcst is not None:
        vFcst = file.createVariable("fcst", "f4", ("time", "leadtime", "location"))
    if obs is not None:
        vObs = file.createVariable("obs", "f4", ("time", "leadtime", "location"))
    if len(args.thresholds) > 0:
        vCdf = file.createVariable("cdf", "f4", ("time", "leadtime", "location", "threshold"))
        vThreshold = file.createVariable("threshold", "f4", ("threshold"))
    if len(args.quantiles) > 0:
        vX = file.createVariable("x", "f4", ("time", "leadtime", "location", "quantile"))
        vQuantile = file.createVariable("quantile", "f4", ("quantile"))
    if args.pit:
        vPit = file.createVariable("pit", "f4", ("time", "leadtime", "location"))
    file.long_name = ifile.variable.name
    file.units = unit = ifile.variable.units.replace("$", "")
    file.Convensions = "verif_1.0.0"
    new_history = ' '.join(sys.argv)
    if hasattr(file, 'history'):
        file.history = file.history + '\n' + new_history
    else:
        file.history = new_history

    if obs is not None:
        vObs[:] = obs
    if fcst is not None:
        vFcst[:] = fcst
    vTime[:] = times
    vOffset[:] = leadtimes
    vLocation[:] = locationids
    vLat[:] = lats
    vLon[:] = lons
    vElev[:] = elevs
    if len(args.thresholds) > 0:
        vThreshold[:] = args.thresholds
        vCdf[:] = cdf
    if len(args.quantiles) > 0:
        vQuantile[:] = args.quantiles
        vX[:] = x
    if args.pit:
        vPit[:] = pit

    file.close()

if __name__ == '__main__':
    main()



from __future__ import print_function
import unittest
import verif.metric
import verif.metric_type
import numpy as np


def get(filenames):
    inputs = [
        verif.input.get_input(file)
        for file in ["verif/tests/files/" + f for f in filenames]
    ]
    data = verif.data.Data(inputs=inputs)
    return data


interval6 = verif.interval.Interval(5.5, 6.5, True, True)
interval100 = verif.interval.Interval(99.5, 100.5, True, True)


class MetricTest(unittest.TestCase):
    def test_func(self):
        metric = verif.metric.Mae()
        metric.aggregator = verif.aggregator.Mean()
        value = metric.compute_from_obs_fcst(np.array([2, 1, 2]), np.array([2, 3, 1]))
        self.assertEqual(value, 1)
        value = metric.compute_from_obs_fcst(np.array([-2]), np.array([2]))
        self.assertEqual(value, 4)
        value = metric.compute_from_obs_fcst(np.array([]), np.array([]))
        self.assertTrue(np.isnan(value))
        value = metric.compute_from_obs_fcst(
            np.array([2, np.nan, 2]), np.array([2, 3, 1])
        )
        self.assertEqual(value, 0.5)

    def test_get_all(self):
        types = verif.metric.get_all()
        self.assertTrue(verif.metric.Mae in [t[1] for t in types])

    def test_get_all_by_type(self):
        types = verif.metric.get_all_by_type(verif.metric_type.Deterministic())
        self.assertTrue(verif.metric.Mae in [t[1] for t in types])

        types = verif.metric.get_all_by_type(float)
        self.assertEqual(len(types), 0)

    def test_get(self):
        metric = verif.metric.get("mae")
        self.assertTrue(isinstance(metric, verif.metric.Mae))

        # with self.assertRaises(ValueError):
        metric = verif.metric.get("not_exist")
        self.assertEqual(metric, None)

    def test_get_all_obs_fcst_based(self):
        types = verif.metric.get_all_obs_fcst_based()
        self.assertTrue(verif.metric.Mae in [t[1] for t in types])

    def test_name(self):
        mae = verif.metric.Mae()
        self.assertEqual("Mean absolute error", mae.name)

    def test_reample(self):
        metric = verif.metric.Ets()

        obs = np.random.randn(10000)
        fcst = np.random.randn(10000)
        othreshold = verif.interval.Interval(0.1, np.inf, True, True)
        fthresholds = [
            verif.interval.Interval(i, np.inf, True, True) for i in np.random.randn(1)
        ]

        # Check that this runs
        q = np.zeros(len(fthresholds), float)
        for i in range(0, len(fthresholds)):
            q[i] = metric.compute_from_obs_fcst_resample(
                obs, fcst, 1, othreshold, fthresholds[i]
            )
        for i in range(0, len(fthresholds)):
            q[i] = metric.compute_from_obs_fcst(obs, fcst, fthresholds[i])

    def test_label(self):
        variable = verif.variable.Variable("test", "C")
        metric = verif.metric.Mae()
        self.assertEqual(metric.label(variable), "MAE (C)")

    def test_is_valid(self):
        metric = verif.metric.Mae()
        self.assertTrue(metric.is_valid())

        metric = verif.metric.Metric()
        self.assertFalse(metric.is_valid())

        metric = verif.metric.XConditional()
        self.assertFalse(metric.is_valid())

    def test_help_and_label(self):
        """Check that all metrics have a help text and a label"""
        variable = verif.variable.Variable("test", "C")
        for name, metric in verif.metric.get_all():
            if metric.is_valid():
                with self.subTest(metric=name):
                    string = metric.help()
                    self.assertTrue(len(string) > 0)
                    metric = metric()
                    label = metric.label(variable)
                    self.assertTrue(len(label) > 0)


class TestObsFcstBased(unittest.TestCase):
    def test_basic(self):
        # Try ObsFcstBased metrics here only
        metrics = [
            verif.metric.Rmse(),
            verif.metric.Mae(),
            verif.metric.Bias(),
            verif.metric.Corr(),
            verif.metric.Dmb(),
            verif.metric.Mbias(),
            verif.metric.Ef(),
        ]
        obs = [[0, 1.5, 2], [0]]  # Case1, Case 2, ...
        fcst = [[3.1, 1.1, -2.1], [1]]
        expected = [
            [2.976575213, 1],  # Rmse
            [2.5333333333, 1],  # Mae
            [-0.466666666666, 1],  # Bias
            [-0.915724295, np.nan],  # Corr
            [1.0 / 0.6, 0],  # Dmb
            [0.6, np.nan],  # Mbias
            [1.0 / 3, 1],  # Ef
        ]

        for m in range(len(metrics)):
            metric = metrics[m]
            for i in range(0, len(obs)):
                value = metric.compute_from_obs_fcst(
                    np.array(obs[i]), np.array(fcst[i])
                )
                if np.isnan(value):
                    self.assertTrue(np.isnan(expected[m][i]))
                else:
                    self.assertAlmostEqual(expected[m][i], value)

    def test_obs_axis(self):
        data = get(["file1.txt"])
        metric = verif.metric.Mae()
        value = metric.compute_single(data, 0, verif.axis.Obs(), 0, interval6)
        # Check that the MAE when obs == 6 is indeed 7
        self.assertEqual(value, 1)

        value = metric.compute_single(data, 0, verif.axis.Obs(), 0, interval100)
        self.assertTrue(np.isnan(value))

    def test_fcst_axis(self):
        data = get(["file1.txt"])
        metric = verif.metric.Mae()
        value = metric.compute_single(data, 0, verif.axis.Fcst(), 0, interval6)
        # Check that the MAE when fcst == 6 is indeed 3
        self.assertEqual(value, 3)

        value = metric.compute_single(data, 0, verif.axis.Fcst(), 0, interval100)
        self.assertTrue(np.isnan(value))


class TestFromField(unittest.TestCase):
    def test_obs_axis(self):
        data = get(["file1.txt"])
        metric = verif.metric.Fcst()
        value = metric.compute_single(data, 0, verif.axis.Obs(), 0, interval6)
        # Check that the average forecast when obs == 6 is indeed 7
        self.assertEqual(value, 7)

    def test_fcst_axis(self):
        data = get(["file1.txt"])
        metric = verif.metric.Mae()
        value = metric.compute_single(data, 0, verif.axis.Fcst(), 0, interval6)
        # Check that the average obs when fcst == 6 is indeed 3
        self.assertEqual(value, 3)


class TestThresholdBased(unittest.TestCase):
    def test_basic(self):
        """Test the basic threshold-based metrics"""
        metrics = [
            verif.metric.Within(),
            verif.metric.A(),  # Hit
            verif.metric.B(),  # FA
            verif.metric.C(),  # Miss
            verif.metric.D(),  # Correct rejection
            verif.metric.Hit(),
            verif.metric.Threat(),
            verif.metric.Conditional(),
            verif.metric.XConditional(func=np.median),
        ]
        intervals = [
            verif.interval.Interval(-np.inf, 0, True, True),  # [-inf, 0]
            verif.interval.Interval(-np.inf, 1, True, True),
            verif.interval.Interval(-np.inf, 2, True, True),
        ]
        obs = [0, 1.5, 2]
        fcst = [3.1, 1.1, -2.1]
        N = len(obs) * 1.0

        # Each line is one metric (one number for each threshold)
        expected = [
            [0 / N, 100 / N, 100 / N],  # Within
            [0 / N, 0 / N, 2 / N],  # Hit
            [1 / N, 1 / N, 0 / N],  # FA
            [1 / N, 1 / N, 1 / N],  # Miss
            [1 / N, 1 / N, 0 / N],  # Correct rejection
            [0, 0, 2.0 / 3],  # Hit rate
            [0, 0, 2.0 / 3],  # Threat score
            [3.1, 3.1, 0.7],  # Average fcst given obs in interval
            [0, 0, 1.5],  # Average obs given obs in interval
        ]

        for m in range(len(metrics)):
            metric = metrics[m]
            for i in range(len(intervals)):
                value = metric.compute_from_obs_fcst(
                    np.array(obs), np.array(fcst), intervals[i]
                )
                ex = expected[m][i] * 1.0
                if np.isnan(value):
                    self.assertTrue(np.isnan(ex))
                else:
                    self.assertAlmostEqual(ex, value)

    def test_threat(self):
        """An extra test for Threat score"""
        metric = verif.metric.Threat()
        obs = np.array([0, 1, 2, 3])
        fcst = np.array([0, 3, 1, 2])

        # Hits: 1
        # FA: 1
        # Miss: 1
        # CR: 0
        interval = verif.interval.Interval(1.5, np.inf, True, True)
        f_interval = verif.interval.Interval(1.5, np.inf, True, True)
        value = metric.compute_from_obs_fcst(obs, fcst, interval, f_interval)
        self.assertEqual(value, 1.0 / 3)


class NameTest(unittest.TestCase):
    def test_valid(self):
        metrics = [x[1] for x in verif.metric.get_all() if x[1].is_valid]
        for metric in metrics:
            self.assertTrue(metric.name != "")
            self.assertTrue(metric.description != "")


class TestBrierScore(unittest.TestCase):
    def test(self):
        """
        These have been checked against the Brier scores in the verification R package. The 'bins'
        attribute was set to False, and the code had to be changed because it didn't work for cases
        where forecasts were not equal to whole 1/10ths (i.e. 0.1, 0.2).

        To give the same results, there can only be one forecasted value within each bin. I.e. 0.21
        and 0.22 cannot appear in the same test case.
        """
        bs = verif.metric.Bs()
        bsrel = verif.metric.BsRel()
        bsres = verif.metric.BsRes()
        bsunc = verif.metric.BsUnc()
        bss = verif.metric.Bss()
        obs = [
            [0],
            [0],
            [0],
            [1],
            [0, 0, 1, 1, 1],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
        ]
        fcst = [
            [0],
            [1],
            [0.3],
            [0.1],
            [0.21, 0.21, 0.21, 0.91, 0.91],
            [
                0.06,
                0.61,
                0.45,
                0.87,
                0.13,
                0.61,
                0.79,
                0.61,
                0.06,
                0.06,
                0.79,
                0.61,
                0.13,
                0.13,
                0.79,
                0.21,
                0.06,
                0.55,
                0.37,
                0.37,
            ],
        ]
        ans = {
            bs: [0, 1, 0.09, 0.81, 0.1457, 0.34928],
            bsrel: [0, 1, 0.09, 0.81, 0.01236667, 0.2076133],
            bsres: [0, 0, 0, 0, 0.1066667, 0.1083333],
            bsunc: [0, 0, 0, 0, 0.24, 0.25],
            bss: [np.nan, np.nan, np.nan, np.nan, 0.3929167, -0.39712],
        }
        for i in range(len(obs)):
            o = np.array(obs[i])
            f = np.array(fcst[i])
            for key in ans:
                calculated = key.compute_from_obs_fcst(o, f)
                expected = ans[key][i]
                if np.isnan(expected):
                    self.assertTrue(np.isnan(expected), np.isnan(calculated))
                else:
                    self.assertAlmostEqual(expected, calculated, places=5)


if __name__ == "__main__":
    unittest.main()



import unittest
import verif.metric_type


class TestMetricType(unittest.TestCase):
    def test_equal(self):
        self.assertEqual(
            verif.metric_type.Deterministic(), verif.metric_type.Deterministic()
        )
        self.assertTrue(
            verif.metric_type.Deterministic() == verif.metric_type.Deterministic()
        )
        self.assertFalse(
            verif.metric_type.Deterministic() != verif.metric_type.Deterministic()
        )

    def test_unequal(self):
        self.assertFalse(
            verif.metric_type.Deterministic() == verif.metric_type.Probabilistic()
        )
        self.assertFalse(
            verif.metric_type.Deterministic() == verif.metric_type.Probabilistic()
        )
        self.assertTrue(
            verif.metric_type.Deterministic() != verif.metric_type.Probabilistic()
        )
        self.assertTrue(
            verif.metric_type.Deterministic() != verif.metric_type.Probabilistic()
        )


if __name__ == "__main__":
    unittest.main()



import sys
import argparse
import numpy as np
import verif.input
import netCDF4
import copy
import scipy.signal


def convolve(array, window, ignore_missing, axis):
    """ Convolves the Verif array across the leadtime dimension """
    assert(len(array.shape) == 3)
    if axis == 'leadtime':
        if window > array.shape[1]:
            verif.util.error("Window (%d) is longer than dimension size (%d)" % (window, array.shape[1]))
        c = np.ones([1, window, 1])
        if ignore_missing:
            # Does this work?
            array[np.isnan(array)] = 0
        new_array = np.nan*np.zeros(array.shape)
        new_array[:, (window-1):, :] = scipy.signal.convolve(array, c, "valid")
    elif axis == 'time':
        if window > array.shape[0]:
            verif.util.error("Window (%d) is longer than dimension size (%d)" % (window, array.shape[0]))
        c = np.ones([window, 1, 1])
        if ignore_missing:
            # Does this work?
            array[np.isnan(array)] = 0
        new_array = np.nan*np.zeros(array.shape)
        new_array[(window-1):, :, :] = scipy.signal.convolve(array, c, "valid")
    else:
        verif.util.error('Invalid axis %s' % axis)
    return new_array


def main():
    parser = argparse.ArgumentParser(prog="accumulate", description="Accumlates a verif file. For each leadtime, sum up values in a specified number of leadtimes leading up to this time. I.e. for a file with hourly time steps, -w 24 creates 24 hour accumulations. For leadtime 36 this is the sum of leadtimes 13-36.")
    parser.add_argument('ifile', help="Verif text or NetCDF file (input)")
    parser.add_argument('ofile', help="Verif NetCDF file (output)")
    parser.add_argument('-w', type=int, help="Accumulation window (in number of timesteps). If omitted, accumulate the whole leadtime axis.", dest="w")
    parser.add_argument('-i', help="Ignore missing values in the sum", action="store_true", dest="ignore")
    parser.add_argument('-x', default="leadtime", help="Axis", choices=["leadtime", "time"], dest="axis")

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(0)

    args = parser.parse_args()

    ifile = verif.input.get_input(args.ifile)
    locations = ifile.locations
    locationids = [loc.id for loc in locations]
    leadtimes = ifile.leadtimes
    times = ifile.times
    lats = [loc.lat for loc in locations]
    lons = [loc.lon for loc in locations]
    elevs = [loc.elev for loc in locations]

    fcst = copy.deepcopy(ifile.fcst)
    obs = copy.deepcopy(ifile.obs)
    if args.axis == 'leadtime':
        axis = 1
    elif args.axis == 'time':
        axis = 0

    if args.w is None:
        if args.ignore:
            fcst = np.nancumsum(fcst, axis=axis)
            obs = np.nancumsum(obs, axis=axis)
        else:
            fcst = np.cumsum(fcst, axis=axis)
            obs = np.cumsum(obs, axis=axis)

    elif args.w > 1:
        # if args.w % 2 == 0:
        #     verif.util.error("Window length has to be an odd number")

        fcst = convolve(fcst, args.w, args.ignore, args.axis)
        obs = convolve(obs, args.w, args.ignore, args.axis)

    file = netCDF4.Dataset(args.ofile, 'w', format="NETCDF4")
    file.createDimension("leadtime", len(ifile.leadtimes))
    file.createDimension("time", None)
    file.createDimension("location", len(ifile.locations))
    vTime = file.createVariable("time", "i4", ("time",))
    vOffset = file.createVariable("leadtime", "f4", ("leadtime",))
    vLocation = file.createVariable("location", "f8", ("location",))
    vLat = file.createVariable("lat", "f4", ("location",))
    vLon = file.createVariable("lon", "f4", ("location",))
    vElev = file.createVariable("altitude", "f4", ("location",))
    vfcst = file.createVariable("fcst", "f4", ("time", "leadtime", "location"))
    vobs =  file.createVariable("obs", "f4", ("time", "leadtime", "location"))
    file.long_name = ifile.variable.name
    file.units = unit = ifile.variable.units.replace("$", "")
    file.Convensions = "verif_1.0.0"

    vobs[:] = obs
    vfcst[:] = fcst
    vTime[:] = times
    vOffset[:] = leadtimes
    vLocation[:] = locationids
    vLat[:] = lats
    vLon[:] = lons
    vElev[:] = elevs

    file.close()

if __name__ == '__main__':
    main()
